
import time
import unittest

from nive.definitions import *
from nive.forms import *
from nive.events import Events
from nive.views import BaseView
from nive.security import User as UserO
from nive.helper import Request
import db_app


data1_1 = { u"ftext": "this is text!",
            u"fnumber": "123456",
            u"fdate": "2008-06-23 16:55:20",#
            u"flist": ["item 1", "item 2", "item 3"],
            u"fmselect": "item 5",
            u"funit": "35",
            u"funitlist": "34\n35\n36",
            u"title":"title data 1",
            u"pool_type": "type1",
            u"pool_category": "cat1"}

data1_2 = { u"ftext": "this is a new text!",
            u"funit": "0",
            u"fdate": "2008-06-23 12:00:00",
            u"title":"new title data 1"}


class Viewy(BaseView):
    def __init__(self):
        self.request = Request()
        self.context = None
    
    def User(self):
        return UserO(u"test")

# -----------------------------------------------------------------
from pyramid import testing

class FormTest(unittest.TestCase):

    def setUp(self):
        self.app = db_app.app_nodb()
        self.view = Viewy()
        request = testing.DummyRequest()
        request._LOCALE_ = "en"
        self.config = testing.setUp(request=request)
    
    def tearDown(self):
        self.app.Close()
        testing.tearDown()
        pass
    

    def test_form(self, **kw):
        form = Form(loadFromType="type1", context=None, request=Request(), app=self.app, view=self.view)
        form.Setup()
        self.assert_(form.GetFields())
        form._SetUpSchema()
        self.assertFalse(form.GetField("test000"))
        request = Request()
        form.LoadDefaultData()
        form.GetActions(True)
        form.GetActions(False)
        form.GetFormValue("test", request=request, method=None)
        form.GetFormValues(request)
        #form.StartForm()
        #form.StartRequestGET()
        #form.Cancel()


    def test_empty(self, **kw):
        form = Form(loadFromType="type1", app=self.app, view=self.view)
        form.Setup()
        result, data = form.Extract({"ftext": ""})
        self.assert_(data.get("ftext")=="")
        
        form = Form(loadFromType="type2", app=self.app, view=self.view)
        form.Setup()
        result, data = form.Extract({"fstr": ""})
        self.assert_(data.get("fstr")=="" and data.get("ftext")==None)

        
    def test_values(self, **kw):
        form = Form(loadFromType="type1", app=self.app, view=self.view)
        form.Setup()

        v,d,e = form.Validate(data=data1_1)
        self.assert_(v, e)
        
        v,d,e = form.Validate(data1_2)
        self.assertFalse(v, e)
        
        result, data = form.Extract(data1_1)
        self.assert_(data)
        result, data = form.Extract(data1_2)
        self.assert_(data)        


    def test_values2(self, **kw):
        form = Form(loadFromType="type1", app=self.app, view=self.view)
        subsets = {"test": {"fields": ["ftext",u"funit",u"title"]}}
        form.subsets = subsets
        form.Setup(subset="test")
        v,d,e = form.Validate(data1_2)
        self.assert_(v, e)

        form = Form(loadFromType="type1", app=self.app, view=self.view)
        subsets = {u"test": {"fields": [u"ftext", FieldConf(**{"id": "section1", "name": "Section 1", "datatype": "section", "fields": [u"funit"]}), u"title"]}}
        form.subsets = subsets
        form.Setup(subset=u"test")
        v,d,e = form.Validate(data1_2)
        self.assert_(v, e)

        result, data = form.Extract(data1_1)
        self.assert_(data)
        result, data = form.Extract(data1_2)
        self.assert_(data)


    def test_html(self, **kw):
        form = HTMLForm(loadFromType="type1", app=self.app, view=self.view, request=Request())
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"
        form.Setup()

        v,d,e = form.Validate(data1_1)
        self.assert_(v, e)
        
        v,d,e = form.Validate(data1_2)
        self.assertFalse(v, e)
        form.Render(d, msgs=None, errors=None)
        
        result, data = form.Extract(data1_1)
        self.assert_(data)
        result, data = form.Extract(data1_2)
        self.assert_(data)
        
        
    def test_html2(self, **kw):
        form = HTMLForm(loadFromType="type1", app=self.app, view=self.view, request=Request())
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"
        form.subsets = {u"test": {"fields": [u"ftext",u"funit",u"title"]}}
        form.Setup(subset=u"test")
        v,d,e = form.Validate(data1_2)
        self.assert_(v, e)
        form.Render(d, msgs=None, errors=None)


    def test_html3(self, **kw):
        form = HTMLForm(loadFromType="type1", app=self.app, view=self.view, request=Request())
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"
        form.subsets = {u"test": {"fields": [u"ftext", FieldConf(**{"id": "section1", "name": "Section 1", "datatype": "section", "fields": [u"funit"]}), u"title"]}}
        form.Setup(subset=u"test")
        v,d,e = form.Validate(data1_2)
        self.assert_(v, e)
        form.Render(d, msgs=None, errors=None)


    def test_tool(self, **kw):
        tool = self.app.GetTool("nive.components.tools.example")
        form = ToolForm(loadFromType=tool.configuration, context=tool,app=self.app, view=self.view, request=Request())
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"
        form.actions = [
            Conf(**{"id": "default", "name": "Speichern", "method": "StartForm", "description": "", "hidden":True, "css_class":"", "html":"", "tag":""}),
            Conf(**{"id": "run",       "name": "Speichern", "method": "RunTool",      "description": "", "css_class":"", "html":"", "tag":""}),
            Conf(**{"id": "cancel",  "name": "Abbrechen", "method": "Cancel",    "description": "", "css_class":"", "html":"", "tag":""})
            ]
        form.Setup()
        data = {"parameter1": "True", "parameter2": "test"}

        form.Process()
        req = {"run$":1}
        req.update(data)
        form.request = req
        form.Process()
        req = {"cancel$":1}
        form.request = req
        form.Process()
        
        form.subsets = {u"test": {"fields": [u"parameter1"]}}
        form.Setup("test")
        form.Process()
        req = {"run$":1}
        req.update(data)
        form.request = req
        form.Process()
        req = {"cancel$":1}
        form.request = req
        form.Process()

        form.subset = "test"
        form.subsets = {u"test": {"fields": [u"parameter1", FieldConf(**{"id": "section1", "name": "Section 1", "datatype": "section", "fields": [u"parameter2"]})]}}
        form.Setup("test")
        form.Process()
        req = {"run$":1}
        req.update(data)
        form.request = req
        form.Process()
        req = {"cancel$":1}
        form.request = req
        form.Process()
        

    def test_json(self, **kw):
        form = JsonMappingForm(request=Request(),app=self.app, view=self.view)
        form.fields = (
            FieldConf(id="parameter1", datatype="text", size=1000),
            FieldConf(id="parameter2", datatype="string", size=100),
        )
        form.Setup()

        data = {"parameter1": "True", "parameter2": "test"}
        r,v,e = form.Validate(data)
        self.assert_(r, e)
        self.assert_(isinstance(v, dict))
        data = {"parameter1": "True", "parameter2": "test"}
        r,v = form.Extract(data)
        self.assert_(r, v)
        self.assert_(isinstance(v, dict))
        

    def test_actions(self, **kw):
        form = HTMLForm(loadFromType="type1", app=self.app, view=self.view, request=Request())
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"

        # create
        form.actions = [
        Conf(**{"id": "default", "method": "StartForm","name": "Initialize", "hidden":True,  "description": "", "css_class":"", "html":"", "tag":""}),
        Conf(**{"id": "create",  "method": "CreateObj","name": "Create",     "hidden":False, "description": "", "css_class":"", "html":"", "tag":""}),
        Conf(**{"id": "cancel",  "method": "Cancel",   "name": "Cancel",     "hidden":False, "description": "", "css_class":"", "html":"", "tag":""})
        ]
        form.Setup()
        a = form.GetActions(removeHidden=False)
        self.assert_(len(a)==3)
        a = form.GetActions(removeHidden=True)
        self.assert_(len(a)==2)


    def test_fields(self, **kw):
        form = HTMLForm(loadFromType="type1", app=self.app, view=self.view, request=Request())
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"
        # create
        form.Setup()

        f = form.GetFields(removeReadonly=True)
        self.assert_(len(f)==8)
        f = form.GetFields(removeReadonly=False)
        self.assert_(len(f)==13)

        

class FormTest_db(unittest.TestCase):

    def setUp(self):
        self.app = db_app.app_db()
        self.request = testing.DummyRequest()
        self.request._LOCALE_ = "en"
        self.config = testing.setUp(request=self.request)
        self.remove=[]
    
    def tearDown(self):
        v = Viewy()
        u = v.User()
        root = self.app.root()
        for r in self.remove:
            root.Delete(r, u)
        self.app.Close()
        testing.tearDown()
        pass
    
    def test_obj(self, **kw):
        root = self.app.GetRoot()
        v = Viewy()
        form = ObjectForm(loadFromType="type1", context=root, view=v, request=Request(), app=self.app)
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"

        # create
        form.actions = [
        Conf(**{"id": "default", "method": "StartForm","name": "Initialize", "hidden":True,  "description": "", "css_class":"", "html":"", "tag":""}),
        Conf(**{"id": "create",  "method": "CreateObj","name": "Create",     "hidden":False, "description": "", "css_class":"", "html":"", "tag":""}),
        Conf(**{"id": "cancel",  "method": "Cancel",   "name": "Cancel",     "hidden":False, "description": "", "css_class":"", "html":"", "tag":""})
        ]
        form.Setup(addTypeField = True)
        
        count = self.app.db.GetCountEntries()
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"default")
        self.assertEqual(count, self.app.db.GetCountEntries())

        req = {"create$":1, "pool_type":"type1"}
        req.update(data1_1)
        form.request = req
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"create")
        self.assertEqual(count+1, self.app.db.GetCountEntries())
        self.remove.append(result.id)
        
        req = {"cancel$":1}
        form.request = req
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"cancel")
        self.assertEqual(count+1, self.app.db.GetCountEntries())
        
        form = ObjectForm(loadFromType="type1", context=root, view=v, request=Request(), app=self.app)
        form.subsets = {u"test": {"fields": [u"ftext",u"funit",u"title"], 
                                  "actions": [u"default", u"create",u"cancel"]}}
        form.Setup(subset = "test", addTypeField = True)
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"default")
        self.assertEqual(count+1, self.app.db.GetCountEntries())

        req = {"create$":1, "pool_type":"type1"}
        req.update(data1_2)
        form.request = req
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"create")
        self.assertEqual(count+2, self.app.db.GetCountEntries())
        self.remove.append(result.id)
        
        req = {"cancel$":1}
        form.request = req
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"cancel")
        self.assertEqual(count+2, self.app.db.GetCountEntries())

        form = ObjectForm(loadFromType="type1", context=root, view=v, request=Request(), app=self.app)
        form.subsets = {u"test": {"fields": [u"ftext", FieldConf(**{"id": "section1", "name": "Section 1", "datatype": "section", "fields": [u"fnumber"]})], 
                                  "actions": [u"default", u"create",u"cancel"]}}
        form.Setup(subset = "test", addTypeField = True)
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"default")
        self.assertEqual(count+2, self.app.db.GetCountEntries())
        
        req = {"create$":1, "pool_type":"type1"}
        req.update(data1_2)
        form.request = req
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"create")
        self.assertEqual(count+3, self.app.db.GetCountEntries())
        self.remove.append(result.id)

        req = {"cancel$":1}
        form.request = req
        result, data, action=form.Process()
        self.assert_(result)
        self.assertEqual(action.id,"cancel")
        self.assertEqual(count+3, self.app.db.GetCountEntries())


    def test_obj2(self, **kw):
        user = UserO(u"test")
        root = self.app.GetRoot()
        obj = root.Create("type1", data1_2, user)
        self.remove.append(obj.id)
        v = Viewy()

        form = ObjectForm(loadFromType="type1", context=obj, view=v, request=Request(), app=self.app)
        form.formUrl = "form/url"
        form.cssID = u"upload"
        form.css_class = u"niveform"
        count = self.app.db.GetCountEntries()
        # edit
        form.actions = [
        Conf(**{"id": "default", "method": "StartObject","name": "Initialize",    "hidden":True, "description": "", "css_class":"", "html":"", "tag":""}),
        Conf(**{"id": "save",       "method": "UpdateObj","name": "Save",         "hidden":False, "description": "", "css_class":"", "html":"", "tag":""}),
        Conf(**{"id": "cancel",  "method": "Cancel",    "name": "Cancel",         "hidden":False, "description": "", "css_class":"", "html":"", "tag":""})
        ]
        form.Setup()
        form.Process()
        req = {"save$":1}
        req.update(data1_1)
        form.request = req
        form.Process()
        req = {"cancel$":1}
        form.request = req
        form.Process()
        d = form.LoadObjData()
        self.assertEqual(count, self.app.db.GetCountEntries())

        form = ObjectForm(loadFromType="type1", context=obj, view=v, request=Request(), app=self.app)
        form.subsets = {u"test": {"fields": [u"ftext",u"funit",u"title"], 
                                  "actions": [u"defaultEdit",u"edit",u"cancel"]}}
        form.Setup(subset = "test")
        form.Process()
        req = {"edit$":1}
        req.update(data1_2)
        form.request = req
        form.Process()
        req = {"cancel$":1}
        form.request = req
        form.Process()
        self.assertEqual(count, self.app.db.GetCountEntries())

        form = ObjectForm(loadFromType="type1", context=obj, view=v, request=Request(), app=self.app)
        form.subsets = {u"test": {"fields": [u"ftext", FieldConf(**{"id": "section1", "name": "Section 1", "datatype": "section", "fields": [u"fnumber"]})],
                                  "actions": [u"defaultEdit",u"edit",u"cancel"]}}
        form.subset = "test"
        form.Setup(subset = "test")
        form.Process()
        req = {"edit$":1}
        req.update(data1_2)
        form.request = req
        form.Process()
        req = {"cancel$":1}
        form.request = req
        form.Process()
        self.assertEqual(count, self.app.db.GetCountEntries())

        form = ObjectForm(loadFromType="type1", context=obj, view=v, request=Request(), app=self.app)
        form.subsets = {u"test": {"fields": [u"ftext", FieldConf(**{"fields": [u"parameter2"], "id": "section1", "name": "Section 1", "datatype": "section"})],
                                  "actions": [u"defaultEdit",u"edit",u"cancel"]}}
        try:
            form.Setup(subset="test3")
            form.Process()
            self.assert_(False)
        except ConfigurationError:
            pass
        self.assertEqual(count, self.app.db.GetCountEntries())


    def test_json(self, **kw):
        user = UserO(u"test")
        root = self.app.GetRoot()
        obj = root.Create("type1", data1_2, user)
        obj.data["fjson"] = {}
        obj.Commit(user=user)
        self.remove.append(obj.id)
        v = Viewy()

        form = JsonMappingForm(request=Request(),app=self.app,context=obj, view=v)
        form.fields = (
            FieldConf(id="parameter1", datatype="string", size=1000),
            FieldConf(id="parameter2", datatype="string", size=100),
        )
        form.jsonDataField = "fjson"
        form.Setup()

        data = {"parameter1": "True", "parameter2": "test"}
        form.request = data
        form.Process()
        
        data["edit$"] = 1
        form.request = data
        form.Process()

        self.assert_(obj.data.get("ftext"))
        


        

if __name__ == '__main__':
    unittest.main()



import itertools
#<removed import peppercorn

from nive.components.reform import decorator
from nive.components.reform import exception
from nive.components.reform import template
from nive.components.reform import widget
from nive.components.reform import schema

class Field(object):
    """ Represents an individual form field (a visible object in a
    form rendering).

    A :class:`reform.form.Field` object instance is meant to last for
    the duration of a single web request. As a result, a field object
    is often used as a scratchpad by the widget associated with that
    field.  Using a field as a scratchpad makes it possible to build
    implementations of state-retaining widgets while instances of
    those widget still only need to be constructed once instead of on
    each request.
    
    *Attributes*

        schema
            The schema node associated with this field.

        widget
            The widget associated with this field.

        order
            An integer indicating the relative order of this field's
            construction to its children and parents.

        oid
            A string incorporating the ``order`` attribute that can be
            used as a unique identifier in HTML code (often for ``id``
            attributes of field-related elements).  An example oid is
            ``reformField0``.

        name
            An alias for self.schema.name
        
        title
            An alias for self.schema.title

        description
            An alias for self.schema.description

        required
            An alias for self.schema.required

        typ
            An alias for self.schema.typ

        children
            Child fields of this field.

        error
            The exception raised by the last attempted validation of the
            schema element associated with this field.  By default, this
            attribute is ``None``.  If non-None, this attribute is usually
            an instance of the exception class
            :exc:`schema.Invalid`, which has a ``msg`` attribute
            providing a human-readable validation error message.

        errormsg
            The ``msg`` attribute of the ``error`` attached to this field
            or ``None`` if the ``error`` attached to this field is ``None``.

        renderer
            The template :term:`renderer` associated with the form.  If a
            renderer is not passed to the constructor, the default reform
            renderer will be used (the :term:`default renderer`).

        counter
            ``None`` or an instance of ``itertools.counter`` which is used
            to generate sequential order-related attributes such as
            ``oid`` and ``order``.

        resource_registry
            The :term:`resource registry` associated with this field.

    *Constructor Arguments*

      ``renderer``, ``counter`` and ``resource_registry`` are accepted
      as explicit keyword arguments to the :class:`reform.Field`.
      These are also available as attribute values.  ``renderer``, if
      passed, is a template renderer as described in
      :ref:`creating_a_renderer`.  ``counter``, if passed, should be
      an :attr:`itertools.counter` object (useful when rendering
      multiple forms on the same page, see
      `http://reformdemo.repoze.org/multiple_forms/
      <http://reformdemo.repoze.org/multiple_forms/>`_.
      ``resource_registry``, if passed should be a widget resource
      registry (see also :ref:`get_widget_resources`).

      If any of these values is ``None`` (their default), suitable
      default values are used in their place.

      The :class:`reform.Field` constructor also accepts *arbitrary*
      keyword arguments.  When an 'unknown' keyword argument is
      passed, it is attached unmolested to the form field as an
      attribute.

      All keyword arguments (explicit and unknown) are also attached to
      all *children* nodes of the field being constructed.

    """

    error = None
    default_renderer = template.default_renderer
    default_resource_registry = widget.default_resource_registry

    def __init__(self, schema, renderer=None, counter=None,
                 resource_registry=None, **kw):
        self.counter = counter or itertools.count()
        self.order = self.counter.next()
        self.oid = 'reformField%s' % self.order
        self.schema = schema
        self.typ = self.schema.typ # required by Invalid exception
        if renderer is None:
            renderer = self.default_renderer
        if resource_registry is None:
            resource_registry = self.default_resource_registry
        self.renderer = renderer
        self.resource_registry = resource_registry
        self.name = schema.name
        self.title = schema.title
        self.description = schema.description
        self.required = schema.required
        self.children = []
        self.__dict__.update(kw)
        for child in schema.children:
            self.children.append(Field(child,
                                       renderer=renderer,
                                       counter=self.counter,
                                       resource_registry=resource_registry,
                                       **kw))

    def __iter__(self):
        """ Iterate over the children fields of this field. """
        return iter(self.children)

    @classmethod
    def set_zpt_renderer(cls, search_path, auto_reload=True,
                         debug=True, encoding='utf-8',
                         translator=None):
        """ Create a :term:`Chameleon` ZPT renderer that will act as a
        :term:`default renderer` for instances of the associated class
        when no ``renderer`` argument is provided to the class'
        constructor.  The arguments to this classmethod have the same
        meaning as the arguments provided to a
        :class:`reform.ZPTRendererFactory`.

        Calling this method resets the :term:`default renderer`.

        This method is effectively a shortcut for
        ``cls.set_default_renderer(ZPTRendererFactory(...))``."""
        cls.default_renderer = template.ZPTRendererFactory(
            search_path,
            auto_reload=auto_reload,
            debug=debug,
            encoding=encoding,
            translator=translator,
            )

    def translate(self, msgid):
        """ Use the translator passed to the renderer of this field to
        translate the msgid into a term.  If the renderer does not have a
        translator, this method will return the msgid."""
        translate = getattr(self.renderer, 'translate', None)
        if translate is not None:
            return translate(msgid)
        return msgid

    @classmethod
    def set_default_renderer(cls, renderer):
        """ Set the callable that will act as a default renderer for
        instances of the associated class when no ``renderer``
        argument is provided to the class' constructor.  Useful when
        you'd like to use an alternate templating system.

        Calling this method resets the :term:`default renderer`.
        """
        cls.default_renderer = staticmethod(renderer)

    @classmethod
    def set_default_resource_registry(cls, registry):

        """ Set the callable that will act as a default
        :term:`resource registry` for instances of the associated
        class when no ``resource_registry`` argument is provided to
        the class' constructor.  Useful when you'd like to use
        non-default requirement to resource path mappings for the
        entirety of a process.

        Calling this method resets the default :term:`resource registry`.
        """
        cls.default_resource_registry = registry

    def __getitem__(self, name):
        """ Return the subfield of this field named ``name`` or raise
        a :exc:`KeyError` if a subfield does not exist named ``name``."""
        for child in self.children:
            if child.name == name:
                return child
        raise KeyError(name)

    def clone(self):
        """ Clone the field and its subfields, retaining attribute
        information.  Return the cloned field.  The ``order``
        attribute of the node is not cloned; instead the field
        receives a new order attribute; it will be a number larger
        than the last renderered field of this set."""
        cloned = self.__class__(self.schema)
        cloned.__dict__.update(self.__dict__)
        cloned.order = cloned.counter.next()
        cloned.oid = 'reformField%s' % cloned.order
        cloned.children = [ field.clone() for field in self.children ]
        return cloned

    @decorator.reify
    def widget(self):
        """ If a widget is not assigned directly to a field, this
        function will be called to generate a default widget (only
        once). The result of this function will then be assigned as
        the ``widget`` attribute of the field for the rest of the
        lifetime of this field. If a widget is assigned to a field
        before form processing, this function will not be called."""
        wdg = getattr(self.schema, 'widget', None)
        if wdg is not None:
            return wdg
        widget_maker = getattr(self.schema.typ, 'widget_maker', None)
        if widget_maker is None:
            widget_maker = widget.default_widget_makers.get(
                self.schema.typ.__class__)
            if widget_maker is None:
                for (cls, wgt) in widget.default_widget_makers.items():
                    if isinstance(self.schema.typ, cls):
                        widget_maker = wgt
                        break
        if widget_maker is None:
            widget_maker = widget.TextInputWidget
        return widget_maker()

    def get_widget_requirements(self):
        """ Return a sequence of two tuples in the form
        [(``requirement_name``, ``version``), ..].

        The first element in each two-tuple represents a requirement
        name.  When a requirement name is returned as part of
        ``get_widget_requirements``, it means that one or more CSS or
        Javascript resources need to be loaded by the page performing
        the form rendering in order for some widget on the page to
        function properly.
        
        The second element in each two-tuple is the reqested version
        of the library resource.  It may be ``None``, in which case
        the version is unspecified.

        See also the ``requirements`` attribute of
        :class:`reform.Widget` and the explanation of widget
        requirements in :ref:`get_widget_requirements`.
        """
        L = []
        requirements = self.widget.requirements
        if requirements:
            for requirement in requirements:
                reqt = tuple(requirement)
                if not reqt in L:
                    L.append(reqt)
        for child in self.children:
            for requirement in child.get_widget_requirements():
                reqt = tuple(requirement)
                if not reqt in L:
                    L.append(reqt)
        return L

    def get_widget_resources(self, requirements=None):
        """ Return a resources dictionary in the form ``{'js':[seq],
        'css':[seq]}``.  ``js`` represents Javascript resources,
        ``css`` represents CSS resources.  ``seq`` represents a
        sequence of resource paths.  Each path in ``seq`` represents a
        relative resource name, as defined by the mapping of a
        requirement to a set of resource specification by the
        :term:`resource registry` attached to this field or form.

        This method may raise a :exc:`ValueError` if the resource
        registry associated with this field or form cannot resolve a
        requirement to a set of resource paths.

        The ``requirements`` argument represents a set of requirements
        as returned by a manual call to
        :meth:`reform.Field.get_widget_requirements`.  If
        ``requirements`` is not supplied, the requirement are implied
        by calling the :meth:`reform.Field.get_widget_requirements`
        method against this form field.

        See also :ref:`get_widget_resources`.
        """
        if requirements is None:
            requirements = self.get_widget_requirements()
        return self.resource_registry(requirements)

    def set_widgets(self, values, separator='.'):
        """ set widgets of the child fields of this field
        or form element.  ``widgets`` should be a dictionary in the
        form::

           {'dotted.field.name':Widget(),
            'dotted.field.name2':Widget()}

        The keys of the dictionary are dotted names.  Each dotted name
        refers to a single field in the tree of fields that are
        children of the field or form object upon which this method is
        called.

        The dotted name is split on its dots and the resulting list of
        names is used as a search path into the child fields of this
        field in order to find a field to which to assign the
        associated widget.

        Two special cases exist:

        - If the key is the empty string (``''``), the widget is
          assigned to the field upon which this method is called.

        - If the key contains an asterisk as an element name, the
          first child of the found element is traversed.  This is most
          useful for sequence fields, because the first (and only)
          child of sequence fields is always the prototype field which
          is used to render all fields in the sequence within a form
          rendering.

        If the ``separator`` argument is passed, it is should be a
        string to be used as the dot character when splitting the
        dotted names (useful for supplying if one of your field object
        has a dot in its name, and you need to use a different
        separator).

        Examples follow.  If the following form is used::

          class Person(Schema):
              first_name = SchemaNode(String())
              last_name = SchemaNode(String())

          class People(SequenceSchema):
              person = Person()

          class Conference(Schema):
              people = People()
              name = SchemaNode(String())

          schema = Conference()
          form = Form(schema)

        The following invocations will have the following results
        against the schema defined above:

        ``form.set_widgets({'people.person.first_name':TextAreaWidget()})``

          Set the ``first_name`` field's widget to a ``TextAreaWidget``.

        ``form.set_widgets({'people.*.first_name':TextAreaWidget()})``

          Set the ``first_name`` field's widget to a
          ``TextAreaWidget``.
        
        ``form.set_widgets({'people':MySequenceWidget()})``

          Set the ``people`` sequence field's widget to a
          ``MySequenceWidget``.

        ``form.set_widgets({'people.*':MySequenceWidget()})``

          Set the *person* field's widget to a ``MySequenceWidget``.

        ``form.set_widgets({'':MyMappingWidget()})``

          Set *form* node's widget to a ``MyMappingWidget``.

        """
        for k, v in values.items():
            if not k:
                self.widget = v
            else:
                path = k.split(separator)
                field = self
                while path:
                    element = path.pop(0)
                    if element == '*':
                        field = field.children[0]
                    else:
                        field = field[element]
                field.widget = v
                
    @property
    def errormsg(self):
        """ Return the ``msg`` attribute of the ``error`` attached to
        this field.  If the ``error`` attribute is ``None``,
        the return value will be ``None``."""
        return getattr(self.error, 'msg', None)

    def serialize(self, cstruct):
        """ Serialize the cstruct into HTML.  """
        return self.widget.serialize(self, cstruct=cstruct)

    def deserialize(self, pstruct, formstruct=None):
        """ Deserialize the pstruct into a cstruct."""
        return self.widget.deserialize(self, pstruct, formstruct)

    def render(self, appstruct=schema.null):
        """ Render the field (or form) to HTML using ``appstruct`` as
        a set of default values.  ``appstruct`` is typically a
        dictionary of application values matching the schema used by
        this form, or ``None``.

        Calling this method is the same as calling::

           cstruct = form.schema.serialize(appstruct)
           form.widget.serialize(field, cstruct)

        See the documentation for
        :meth:`schema.SchemaNode.serialize` and
        :meth:`reform.widget.Widget.serialize` .
        """
        cstruct = self.schema.serialize(appstruct)
        return self.serialize(cstruct)

    def validate(self, controls):
        """
        Validate the set of controls returned by a form submission
        against the schema associated with this field or form.
        ``controls`` should be a *document-ordered* sequence of
        two-tuples that represent the form submission data.  Each
        two-tuple should be in the form ``(key, value)``.  ``node``
        should be the schema node associated with this widget.

        For example, using WebOb, you can compute a suitable value for
        ``controls`` via::

          request.POST.items()

        Or, if you're using a ``cgi.FieldStorage`` object named
        ``fs``, you can compute a suitable value for ``controls``
        via::

          controls = []
          if fs.list:
              for control in fs.list:
                  if control.filename:
                      controls.append((control.name, control))
                  else:
                      controls.append((control.name, control.value))

        Equivalent ways of computing ``controls`` should be available to
        any web framework.

        When the ``validate`` method is called:

        - if the fields are successfully validated, a data structure
          represented by the deserialization of the data as per the
          schema is returned.  It will be a mapping.

        - If the fields cannot be successfully validated, a
          :exc:`reform.exception.ValidationFailure` exception is raised.

        The typical usage of ``validate`` in the wild is often
        something like this (at least in terms of code found within
        the body of a :mod:`pyramid` view function, the particulars
        will differ in your web framework)::

          from webob.exc import HTTPFound
          from nive.components.reform.exception import ValidationFailure
          from nive.components.reform import schema
          from nive.components.reform.form import Form

          from my_application import do_something

          class MySchema(schema.MappingSchema):
              color = schema.SchemaNode(schema.String())

          schema = MySchema()
          form = Form(schema)
          
          if 'submit' in request.POST:  # the form submission needs validation
              controls = request.POST.items()
              try:
                  deserialized = form.validate(controls)
                  do_something(deserialized)
                  return HTTPFound(location='http://example.com/success')
              except ValidationFailure, e:
                  return {'form':e.render()}
          else:
              return {'form':form.render()} # the form just needs rendering
        """
        pstruct = controls
        if not isinstance(pstruct, dict):
            pstruct = {}
            for v in controls:
                pstruct[v[0]]=v[1]
        #<removed pstruct = peppercorn.parse(controls)
        e = None

        try:
            cstruct = self.deserialize(pstruct)
        except schema.Invalid, e:
            # fill in errors raised by widgets
            self.widget.handle_error(self, e)
            cstruct = e.value

        try:
            appstruct = self.schema.deserialize(cstruct)
        except schema.Invalid, e:
            # fill in errors raised by schema nodes
            self.widget.handle_error(self, e)

        if e:
            raise exception.ValidationFailure(self, cstruct, e)

        return appstruct

    def __repr__(self):
        return '<%s.%s object at %d (schemanode %r)>' % (
            self.__module__,
            self.__class__.__name__,
            id(self),
            self.schema.name,
            )



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Nive cms toolbox and editor view layer
----------------------------------------
The view configuration includes a list of assets (javascript and css) needed to render the toolbox
and editor widgets. These are included in the design/main template in edit mode through `cmsview.Assets()`.
Asset definitions use a identifier and an asset path like: ::

    ('jquery.js', 'nive.cms.cmsview:static/mods/jquery.min.js'),
    ('toolbox.css', 'nive.cms.cmsview:static/toolbox/toolbox.css'),

If for example jquery is already included in the main page Assets() can be told to ignore certain
entries: ::

    cmsview.Assets(ignore=["jquery.js"])  

"""
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO

from pyramid.response import Response
from pyramid.renderers import get_renderer, render, render_to_response

    
from nive.i18n import _, translator
from nive.definitions import ViewModuleConf, ViewConf, WidgetConf, FieldConf
from nive.definitions import IContainer, IApplication, IPortal, IPage, IObject, IRoot
from nive.definitions import IToolboxWidgetConf, IEditorWidgetConf, IViewModuleConf, ICMSRoot, IColumn
from nive.utils.utils import SortConfigurationList
from nive.helper import ResolveName

from nive.forms import ObjectForm
from nive.views import BaseView

from nive.cms.cmsview import cutcopy
from nive.cms.cmsview import sort



# view module definition ------------------------------------------------------------------

#@nive_module
configuration = ViewModuleConf(
    id = "editor",
    name = u"CMS Editor",
    static = "nive.cms.cmsview:static",
    templates = "nive.cms.cmsview:",
    permission = "read",
    context = IObject,
    containment = ICMSRoot,  
    view = "nive.cms.cmsview.view.Editor",
    assets = [
        # jquery and jquery-ui
        ('jquery.js', 'nive.cms.cmsview:static/mods/jquery.min.js'),
        ('jquery-ui.js', 'nive.cms.cmsview:static/mods/ui/jquery-ui-1.8.24.custom.min.js'),
        # nive specific
        ('cmseditor.js', 'nive.cms.cmsview:static/cmseditor.js'),
        ('toolbox.css', 'nive.cms.cmsview:static/toolbox/toolbox.css'),
        ('overlay.css', 'nive.cms.cmsview:static/overlay/overlay.css'),
        ('cmseditor.css', 'nive.cms.cmsview:static/cmseditor.css'),
    ],
    description=__doc__
)

# views -----------------------------------------------------------------------------
# copy and sort extension views are imported from their files and added at the end of the list

t = configuration.templates 
configuration.views = [
    ViewConf(name="editor",     attr="editor",  context=IContainer,   permission="read", containment=IApplication),
    ViewConf(name="exiteditor", attr="exit",    context=IContainer,   permission="view", containment=IApplication),
    ViewConf(name="exiteditor", attr="exitapp", context=IApplication, permission="view", containment=IPortal),
    
    #ViewConf(id="rootview", name = "",     attr = "view", context = ICMSRoot, containment=IApplication),
    #ViewConf(id="objview",  name = "",     attr = "view", context = IPage),
    
    # object
    ViewConf(name="edit", attr="edit", renderer=t+"edit.pt", permission="edit"),
    ViewConf(name="meta", attr="meta", renderer=t+"meta.pt"),
    ViewConf(name="delfile", attr="delfile", permission="delete"),
                
    # widgets
    ViewConf(name = "elementListWidget", attr = "elementListWidget", context = IContainer, permission="edit"),
    ViewConf(name = "elementAddWidget",  attr = "elementAddWidget",  context = IObject, permission = "add"),
    ViewConf(name = "elementAddWidget",  attr = "elementAddWidget",  context = IRoot, permission = "add"),
    ViewConf(name = "editblock",         attr = "editBlockElement",  context = IObject, permission = "edit"),
    
    # container
    ViewConf(name = "add",       attr = "add",    context = IContainer, renderer = t+"add.pt", permission="add"),
    ViewConf(name = "delete",    attr = "delete", context = IContainer, renderer = t+"delete.pt", permission = "delete"),
    
    # widgets
    ViewConf(name = "addpageWidget",  attr = "tmpl", renderer = t+"widgets/widget_addpage.pt",    context = IContainer, permission="add"),
    ViewConf(name = "editpageWidget", attr = "tmpl", renderer = t+"widgets/widget_editpage.pt",   context = IContainer, permission="edit"),
    ViewConf(name = "subpagesWidget", attr = "tmpl", renderer = t+"widgets/widget_subpages.pt",   context = IContainer),
    ViewConf(name = "settingsWidget", attr = "tmpl", renderer = t+"widgets/widget_settings.pt",   context = IContainer)
] + sort.views + cutcopy.views


# toolbox and editor widgets ----------------------------------------------------------------------------------
configuration.widgets = [
    WidgetConf(name=_(u"Add new page"),         widgetType=IToolboxWidgetConf, apply=(IContainer,), viewmapper="addpageWidget", id="cms.addpage", sort=100),
    WidgetConf(name=_(u"Edit page"),            widgetType=IToolboxWidgetConf, apply=(IContainer,), viewmapper="editpageWidget", id="cms.editpage", sort=200),
    WidgetConf(name=_(u"Sub pages and parent"), widgetType=IToolboxWidgetConf, apply=(IContainer,), viewmapper="subpagesWidget", id="cms.subpages", sort=300),
    WidgetConf(name=_(u"Settings"),             widgetType=IToolboxWidgetConf, apply=(IApplication,IContainer), viewmapper="settingsWidget", id="cms.settings", sort=400),
    
    WidgetConf(name=_(u"Edit"),          widgetType=IEditorWidgetConf, apply=(IObject,),    viewmapper="edit",   id="editor.edit", sort=100),
    WidgetConf(name=_(u"Add"),           widgetType=IEditorWidgetConf, apply=(IContainer,), viewmapper="add",    id="editor.add",  sort=200),
    WidgetConf(name=_(u"Sort sub pages"),widgetType=IEditorWidgetConf, apply=(IPage,),      viewmapper="sortpages", id="editor.sortpages", sort=300),
    WidgetConf(name=_(u"Meta"),          widgetType=IEditorWidgetConf, apply=(IObject,),    viewmapper="meta",   id="editor.meta", sort=400)
]


        
        
# view implementation ------------------------------------------------------------------
        

class Editor(BaseView, cutcopy.CopyView, sort.SortView):

    def __init__(self, context, request):
        BaseView.__init__(self, context, request)
        request.editmode = "editmode"

    def IsEditmode(self):
        return True
        
    def GetEditorWidgets(self, object):
        app = self.context.app
        widgets = app.QueryConf(IEditorWidgetConf, context=object)
        confs = []
        if not widgets:
            return confs
        #opt
        for n,w in widgets:
            confs.append(w)
        return SortConfigurationList(confs, u"sort")
        
    def Assets(self, assets=None, ignore=None):
        """
        Renders a list of static ressources as html <script> and <link>.
        If assets is None the list of assets is looked up in the 'cmsview'-configuration.
        Asset definitions use a identifier and an asset path like: ::
        
            ('jquery.js', 'nive.cms.cmsview:static/mods/jquery.min.js'),
            ('toolbox.css', 'nive.cms.cmsview:static/toolbox/toolbox.css'),
            ('jquery-ui.js', 'http://example.com/static/mods/jquery.min.js'),
        
        If for example jquery is already included in the main page Assets() can be told to ignore certain
        entries: ::
        
            cmsview.Assets(ignore=["jquery.js"])  
        
        """
        if assets==None:
            app = self.context.app
            conf = app.QueryConfByName(IViewModuleConf, "editor")
            if not conf:
                return u""
            assets = conf.assets

        if not assets:
            return u""
        
        if ignore==None:
            ignore = []

        js_links = [r[1] if r[1].startswith(u"http://") else self.StaticUrl(r[1]) for r in filter(lambda v: v[0] not in ignore and v[1].endswith(u".js"), assets)]
        css_links = [r[1] if r[1].startswith(u"http://") else self.StaticUrl(r[1]) for r in filter(lambda v: v[0] not in ignore and v[1].endswith(u".css"), assets)]
        js_tags = [u'<script src="%s" type="text/javascript"></script>' % link for link in js_links]
        css_tags = [u'<link href="%s" rel="stylesheet" type="text/css" media="all"/>' % link for link in css_links]
        return (u"\r\n").join(js_tags + css_tags)
        
    # macros ------------------------------------------------------------------------ 

    def cmsIndex_tmpl(self):
        i = get_renderer('nive.cms.cmsview:index.pt').implementation()
        return i
    
    # redirects ------------------------------------------------------------------------ 

    def editor(self):
        # switch to editor mode
        root = self.context.app.root("editor")
        url = self.FolderUrl(root)
        if not self.context.IsRoot():
            url = url + self.PageUrl(self.context)[len(self.FolderUrl(self.context.GetRoot())):]
        self.Redirect(url)
    
    def exit(self):
        # leave editor mode
        root = self.context.app.root()
        url = self.FolderUrl(root)
        if not self.context.IsRoot():
            url = url + self.PageUrl(self.context)[len(self.FolderUrl(self.context.root())):]
        self.Redirect(url)

    def exitapp(self):
        # leave editor mode in application context
        root = self.context.root()
        url = self.FolderUrl(root)
        self.Redirect(url)
    
    
    # cms editor interface elements -------------------------------------------------
    
    def cms(self, obj=None):
        """
        Renders required widgets and toolbar
        
        Calls
        - self.cmsToolbox() 
        - self.cmsEditorBlocks()
        - self.cmsEditorBlocks() for each column
        """
        if not obj:
            obj=self.context
        html = [self.cmsToolbox(obj), self.cmsEditorBlocks(obj)]
        for column in obj.app.configuration.columns:
            html.append(self.cmsEditorBlocks(obj.page.GetColumn(column)))
        return u"".join(html)

    
    def cmsToolbox(self, obj, elements=None):
        """
        nive toolbox widget.
        call with obj = current object / page
        """
        if not obj:
            obj = self.context
        return render("widgets/toolbox.pt", {u"obj":obj, u"view":self, u"elements": elements}, request=self.request)

    
    def cmsEditorBlocks(self, obj, elements=None):
        """
        Renders javascript needed to load editblocks of page elements. Editblocks are loaded on click.  
        call with obj = current container / page
        """
        if not obj:
            return u""
        js = u"""<script>$(document).ready(function(){ \n%(js)s });</script>"""
        attr = u""" $("#nive-element%(id)s").click(function() { $.fn.editblocks().clickAndLoadElement('%(id)s','%(path)s',arguments[0] || window.event); });\n"""
        newjs = StringIO()
        html = StringIO()
        
        if not elements:
            elements = obj.GetPageElements()
        
        for el in elements:
            if el.GetTypeID()=='box':
                newjs.write(attr % {u"id": unicode(el.GetID()), u"path": self.FolderUrl(el)})
                for elb in el.GetPageElements():
                    newjs.write(attr % {u"id": unicode(elb.GetID()), u"path": self.FolderUrl(elb)})
            else:
                newjs.write(attr % {u"id": unicode(el.GetID()), u"path": self.FolderUrl(el)})
        
        html.write(js % {u"js": newjs.getvalue()})
        return html.getvalue()


    def cmsEditorBlocksPrerender(self, obj, elements=None):
        """
        Renders javascript needed to load editblocks of page elements. Editblocks are rendered directly into the page.  
        call with obj = current container / page
        """
        if not obj:
            return u""
        js = u"""<script>$(document).ready(function(){ %(js)s });</script>"""
        #dbl click: attr = u""" $("#nive-element%(id)s").attr({ondblclick:"$.fn.editblocks().dblClickElement('%(id)s',event)", onclick:"$.fn.editblocks().clickElement('%(id)s',event)"});\n"""
        attr = u""" $("#nive-element%(id)s").click(function() { $.fn.editblocks().clickElement('%(id)s',arguments[0] || window.event); });\n"""
        insert = u""" $("#nive-editblock%(id)s").prependTo("#nive-element%(id)s");\n"""
        newjs = StringIO()
        html = StringIO()
        
        if not elements:
            elements = obj.GetPageElements()
        
        for el in elements:
            if el.GetTypeID()=='box':
                html.write(self.editBlockElement(obj=el, addResponse=False))
                newjs.write(insert % {u"id": unicode(el.GetID())})
                newjs.write(attr % {u"id": unicode(el.GetID())})
                for elb in el.GetPageElements():
                    html.write(self.editBlockElement(obj=elb, addResponse=False))
                    newjs.write(insert % {u"id": unicode(elb.GetID())})
                    newjs.write(attr % {u"id": unicode(elb.GetID())})
        
            else:
                html.write(self.editBlockElement(obj=el, addResponse=False))
                newjs.write(insert % {u"id": unicode(el.GetID())})
                newjs.write(attr % {u"id": unicode(el.GetID())})
        
        html.write(js % {u"js": newjs.getvalue()})
        return html.getvalue()


    def editBlockPage(self, page=None):
        """
        Edit bar for page main content area or columns
        if page is None current context is used
        """
        if not page:
            page=self.context
        return render("widgets/editblock_page.pt", {u"obj":page, u"view":self}, request=self.request)

    
    def editBlockElement(self, obj=None, addResponse=True):
        """
        Edit bar for elements
        if obj is None current context is used
        """
        if not obj:
            obj=self.context
        data = render("widgets/editblock_element.pt", {u"obj":obj, u"view":self}, request=self.request)
        if addResponse:
            r = Response(content_type="text/html", conditional_response=True)
            r.unicode_body = data
            return r
        return data


    def editBlockColumn(self, name=None, page=None, column=None):
        """
        Edit bar for columns
        if column is None the current context is used
        """
        if not page:
            page = self.context.page
        if not column:
            column=self.context
            if not IColumn.providedBy(column):
                if name:
                    column = page.GetColumn(name)
        if column != None:
            name = column.meta.get("title")
        return render("widgets/editblock_column.pt", 
                      {u"column":column, u"page": page, u"name": name, u"view":self}, 
                      request=self.request)

    
    def editBlockList(self, obj=None, page=None, showCCP=False):
        """
        Edit bar used in lists
        call with obj = current object / page
        """
        if not obj:
            obj=self.context
        if not page:
            page = obj.GetPage()
            elementContainer = obj.GetElementContainer()
        else:
            elementContainer = page
        return render("widgets/editblock_list.pt", 
                      {u"obj":obj, u"page": page, u"elementContainer": elementContainer, u"view":self, u"showCCP":showCCP}, 
                      request=self.request)

    
    def elementAddWidget(self, obj=None, addResponse=True):
        """
        Widget with links to add elements as subobjects of obj
        call with obj = current object / element
        """
        if not obj:
            obj=self.context
        if addResponse:
            return render_to_response("widgets/element_add_list.pt", {"obj":obj, "view":self}, request=self.request)
        return render("widgets/element_add_list.pt", {u"obj":obj, u"view":self}, request=self.request)


    def elementListWidget(self, obj=None, elements=None, addResponse=True):
        """
        Widget with existing elements list and edit options
        call with obj = current object / page
        """
        #i18n?
        if not obj:
            obj=self.context
        html = u"""<div>
  <h4 onclick="$.fn.editblocks().toggleBlock('#elements%(id)s',event)">%(title)s</h4>
  %(blocks)s
</div>
        """
        
        elHtml = u"""<div class="element">
  <div class="el_title">%(title)s</div>
  <div class="el_options">%(options)s</div>
  <br style="clear:both"/>
</div>"""
        
        if not elements:
            elements = obj.GetPageElements()
            
        localizer = translator(self.request)
        
        blocks = StringIO()
        static = self.StaticUrl("nive.cms.cmsview:static/images/types/")
        for el in elements:
        
            t = el.GetTitle()
            if not t:
                t = u"<em>%s</em>" % (localizer(el.GetTypeName(), self.request))

            if el.GetTypeID()=="box":
                title = u"<img src='%s%s.png' align='top'/> %s: %s" % (static, el.GetTypeID(), localizer(u"Box"), t)
                blocks.write(elHtml % {u"title": title, u"options": self.editBlockList(obj=el, showCCP=True)})
                for elb in el.GetPageElements():
                    t = elb.GetTitle()
                    if not t:
                        t = u"<em>%s</em>" % (localizer(elb.GetTypeName()))
                    title = u"&gt; <img src='%s%s.png' align='top'/> %s" % (static, elb.GetTypeID(), t)
                    blocks.write(elHtml % {u"title": title, u"options": self.editBlockList(obj=elb, showCCP=True)})
        
            else:
                title = u"<img src='%s%s.png' align='top'/> %s" % (static, el.GetTypeID(), t)
                blocks.write(elHtml % {u"title": title, u"options": self.editBlockList(obj=el, showCCP=True)})
        if not len(elements):
            blocks.write(localizer(_(u"<p><i>empty</i></p>")))
        data = html % {u"blocks": blocks.getvalue(), u"id": str(obj.GetID()), u"title": localizer(_(u"Page elements"))}
        if addResponse:
            r = Response(content_type="text/html", conditional_response=True)
            r.unicode_body = data
            return r
        return data
        

    def pageListWidget(self, page=None, pages=None):
        """
        Widget with existing pages list and edit options
        call with page = current page
        """
        if not page:
            page=self.context
        html = u"""<div class="subpages"> %(blocks)s </div>"""
        
        pHtml = u"""<div class="element">
  <div class="el_title">%(workflow)s<a href="%(url)s">%(title)s </a> </div>
  <div class="el_options">%(options)s</div>
  <br style="clear:both"/>
</div>"""
        
        useworkflow = 1
        localizer = translator(self.request)
        static = self.StaticUrl("nive.cms.workflow:static/exclamation.png")

        if not pages:
            pages = page.GetPages(includeMenu=1)
        blocks = StringIO()
        for p in pages:
            wf = u""
            if useworkflow and not p.meta.pool_state:
                wf = u"""<a href="%(url)sworkflow" class="right" rel="niveOverlay"><img src="%(static)s" title="%(name)s"/></a>""" % {
                                    u"static": static, 
                                    u"url": self.FolderUrl(p), 
                                    u"name": localizer(_(u"This page is not public."))
                                }
            title = p.meta.get(u"title")
            options = self.editBlockList(obj=p, page=page)
            blocks.write(pHtml % {u"url": self.FolderUrl(p), u"title": title, u"options": options, u"workflow": wf})
        if not len(pages):
            blocks.write(localizer(_(u"<p><i>no sub pages</i></p>")))
        return html % {u"blocks": blocks.getvalue()}
        

    def breadcrumbs(self, addHome=0, link=True):
        """
        """
        base = self.context #.GetPage()
        parents = base.GetParents()
        parents.reverse()
        if not addHome:
            parents = parents[1:]
        if len(parents)==0:
            return u""
        html = StringIO()
        for page in parents:
            title = page.GetTitle() or page.GetTypeName()
            if not link:
                html.write(u"""<span>%s</span> &gt; """ % (title))
            else:
                html.write(u"""<a href="%s" class="nivecms">%s</a> &gt; """ % (self.PageUrl(page), title))
        title = base.GetTitle() or base.GetTypeName()
        if not link:
            html.write(u"""<span>%s</span>""" % (title))
        else:
            html.write(u"""<a href="%s" class="nivecms">%s</a>""" % (self.PageUrl(base), title))
        return html.getvalue()
    
    
    def insertMessages(self):
        messages = self.request.session.pop_flash("")
        if not messages:
            return u""
        html = u"""<div class="alert"> <a href="#" class="close" data-dismiss="alert">&times;</a> <ul><li>%s</li></ul></div>"""
        try:
            return html % (u"</li><li>".join(messages))
        except:
            return u""
        
    def insertPageWidgets(self):
        return self.insertToolboxWidgets(self.context.GetPage())
        
    def insertAppWidgets(self):
        return self.insertToolboxWidgets(self.context.app)

    def insertToolboxWidgets(self, object):
        app = self.context.app
        widgets = app.QueryConf(IToolboxWidgetConf, context=object)
        html = u""
        if not widgets:
            return html
        l = []
        #opt
        for n,w in widgets:
            l.append({u"id":w.sort, u"data": self.RenderView(object, name=w.viewmapper, secure=True, raiseUnauthorized=False)})
        for i in SortConfigurationList(l, u"id"):
            if i[u"data"]:
                html += i[u"data"]
        return html


    # add list widgets -------------------------------------------------------------------

    def selectType(self):
        user = self.User()
        lt = self.context.GetAllowedTypes(user)
        tmpl = u"""<a href="add?pool_type=%s" rel="niveOverlay" class="nivecms addlink">%s</a> """
        html = StringIO()
        html.write(u"""<div class="addElements">""")
        #opt
        for t in lt:
            html.write(tmpl % (t[u"id"], _(t[u"name"])))
        html.write(u"</div>")
        return html.getvalue()
    
    def selectPageElement(self):
        user = self.User()
        lt = self.context.GetAllowedTypes(user)
        tmpl = u"""<a href="add?pool_type=%s" rel="niveOverlay" class="nivecms addlink">%s</a> """
        html = StringIO()
        html.write(u"""<div class="addElements">""")
        #opt
        for t in lt:
            html.write(tmpl % (t[u"id"], _(t[u"name"])))
        html.write(u"</div>")
        return html.getvalue()
    


    # template rendering -----------------------------------------------------------------------
    
    @property
    def editorview(self):
        return self
    
    @property
    def designview(self):
        """
        Tries to load the editor view class. If none is registered the function 
        will simply return None. Otherwise the editor view class instance with 
        context and request set.
        """
        if hasattr(self, "_c_design"):
            return self._c_design
        module = self.context.app.QueryConfByName(IViewModuleConf, "design")
        if not module:
            return None
        cls =  ResolveName(module.view)
        design = cls(self.context, self.request)
        self._c_design = design
        return design
        
    
    def add(self):
        typeID = self.GetFormValue("pool_type")
        if not typeID:
            return {u"content": u"", u"showAddLinks": True, u"result": True, u"head": u""}
        form = ObjectForm(view=self, loadFromType=typeID)
        form.Setup(subset="create", addTypeField=True)
        form.use_ajax = True
        head = form.HTMLHead()
        result, data, action = form.Process(redirectSuccess="page_url")
        return {u"content": data, u"result": result, u"cmsview": self, u"showAddLinks": False, u"head": head}

    
    def delete(self):
        id = self.GetFormValue(u"id")
        result = {u"msgs": [], u"objToDelete": None, u"content":u"", u"cmsview": self, u"result": False}
        if not id:
            result[u"msgs"] = [_(u"Nothing to delete")]
            return result
        delete = self.GetFormValue(u"delete")
        obj = self.context.obj(id)
        if not obj:
            result[u"msgs"] = [_(u"Object not found")]
            return result
        if obj.IsContainer() and delete != u"1":
            result[u"objToDelete"] = obj
            return result
        result[u"result"] = self.context.Delete(id, user=self.User(), obj=obj)
        if result[u"result"]:
            result[u"msgs"] = [_(u"OK. Deleted.")]
        self.Relocate("""<script type="text/javascript">window.parent.close("%s");</script>""" % (self.PageUrl()), 
                      [_(u"OK. Deleted.")], 
                      raiseException=True)
        return result
    
    
    def edit(self):
        form = ObjectForm(view=self, loadFromType=self.context.configuration)
        form.use_ajax = True
        form.Setup(subset="edit")
        head = form.HTMLHead()
        result, data, action = form.Process(redirectSuccess="page_url")
        return {u"content": data, u"result": result, u"cmsview":self, u"head": head}


    def delfile(self):
        file = self.GetFormValue(u"fid")
        try:
            r=self.context.DeleteFile(file, self.User())
            if not r:
                m=_(u"Delete failed")
            else:
                m=_(u"OK")
        except Exception, e:
            m=str(e)
        r = Response(content_type="text/html", conditional_response=True)
        r.text = """<span>%(msg)s</span>""" % {"name": str(file), "msg":m}
        return r


    def meta(self):
        return {}


    # bw 0.9.4 ----------------------------------------------------------------------------
    def cmsMain(self, obj, elements=None):
        return self.cmsToolbox(obj, elements=elements)



import time
import unittest

from nive.definitions import *
from nive.components.tools.dbSqldataDump import *

from nive.tests import db_app

from nive.helper import FormatConfTestFailure

# -----------------------------------------------------------------

class DBSqlDataTest1(unittest.TestCase):

    def test_conf1(self):
        r=configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_tool(self):
        dbSqldataDump(configuration,None)
        
    
class DBSqlDataTest1_db(unittest.TestCase):

    def setUp(self):
        self.app = db_app.app_db()
        self.app.Register(configuration)

    def tearDown(self):
        self.app.Close()
    
    def test_toolrun1(self):
        t = self.app.GetTool("dbSqldataDump", self.app)
        self.assert_(t)
        t.importWf = 0
        t.importSecurity = 0
        r,v = t()
        #print v
        self.assert_(r)


    def test_toolrun2(self):
        t = self.app.GetTool("dbSqldataDump", self.app)
        self.assert_(t)
        t.importWf = 1
        t.importSecurity = 1
        r,v = t()
        self.assert_(r)


if __name__ == '__main__':
    unittest.main()




import time
import unittest
from types import DictType

from nive.definitions import implements
from nive.security import User, AdminUser, GetUsers, Unauthorized, UserFound, IAdminUser
from nive.security import effective_principals
from nive.tests import db_app


class securityTest(unittest.TestCase):

    def test_excps(self):
        u = Unauthorized()
        f = UserFound("user")

    def test_iface(self):
        class aaaaa(object):
            implements(IAdminUser)
        a = aaaaa()

    def test_init1(self):
        app = db_app.app_db()
        GetUsers(app)
    
    def test_user(self):
        u = User("name", 123)
        self.assert_(str(u)=="name")
        self.assert_(u.id==123)
        u.GetGroups()

    def test_adminuser(self):
        u = AdminUser({"name":"admin", "password":"11111", "email":"admin@aaa.ccc", "groups":("group:admin",)}, "admin")
        
        self.assert_(u.identity=="admin")
        self.assert_(str(u)=="admin")
        self.assert_(u.Authenticate("11111"))
        self.assertFalse(u.Authenticate("aaaaaaa"))
        u.Login()
        u.Logout()
        self.assert_(u.GetGroups()==("group:admin",))
        self.assert_(u.InGroups("group:admin"))
        self.assertFalse(u.InGroups("group:traa"))
        self.assert_(u.ReadableName()=="admin")
        
    def test_principals(self):
        p = effective_principals()
        self.assert_(p==None)



                
        

if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
File
----
The *file* element provides a simple file download with title and optional description.
"""

from nive.i18n import _
from nive.utils.utils import FormatBytesForDisplay

from nive.definitions import StagPageElement, ObjectConf, FieldConf
from nive.components.objects.base import PageElementFileBase


class file(PageElementFileBase):
    
    def GetDownloadTitle(self):
        # title with type and size
        title = self.meta["title"]
        f = self.files.get("file")
        if not f:
            return title
        t = f["extension"]
        s = f["size"]
        if s:
            s = FormatBytesForDisplay(s)
        return u"%s (%s %s)" % (title, t, s)
    
     
    def Init(self):
        self.ListenEvent("commit", "FilenameToTitle")

    def FilenameToTitle(self):
        if self.files.get("file") and not self.meta["title"]:
            self.meta["title"] = self.files.get("file")["filename"]
        


# file type definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "file",
    name = _(u"File"),
    dbparam = "files",
    context = "nive.cms.file.file",
    template = "file.pt",
    selectTag = StagPageElement,
    icon = "nive.cms.cmsview:static/images/types/file.png",
    description = _(u"The file element provides a simple file download with title and optional description.")
)

configuration.data = [
    FieldConf(id="file",       datatype="file",  size=0,     default=u"", fulltext=1, name=_(u"File"), description=u""),
    FieldConf(id="textblock",  datatype="htext", size=10000, default=u"", name=_(u"Description"), fulltext=1, description=u"")
]

fields = ["title", "file", "textblock", "pool_groups"]
configuration.forms = {"create": {"fields":fields}, "edit": {"fields":fields}}

configuration.views = []



    



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
This file provides *container* functionality for :term:`objects` and :term:`roots`. The different components
are separated into classes by functionality. 

See :py:mod:`nive.components.objects.base` for subclassing containers.
"""

from time import time
from datetime import datetime

from nive.definitions import Conf
from nive.definitions import StagContainer, StagPageElement, MetaTbl
from nive.definitions import IContainer, ICache, IObject, IConf 
from nive.definitions import ContainmentError, ConfigurationError
from nive.workflow import WorkflowNotAllowed
from nive.helper import ResolveName, ClassFactory


class Container(object):
    """
    Container implementation with read access for subobjects used for objects and roots.

    Requires: ContainerFactory
    """

    def obj(self, id, **kw):
        """ shortcut for GetObj """
        return self.GetObj(id, **kw)

    def __getitem__(self, id):
        id = id.split(".")[0]
        o = self.GetObj(id)
        if o:
            return o
        raise KeyError, id


    # Container functions ----------------------------------------------------

    def GetObj(self, id, **kw):
        """
        Get the subobject by id. ::
        
            id = object id as number
            **kw = load information
            returns the object or None
        
        Events:
        - loadObj(obj)
        """
        try:
            id = long(id)
        except:
            return None
        if not id:
            return None
        obj = self._GetObj(id, **kw)
        self.Signal("loadObj", obj)
        return obj


    def GetObjs(self, parameter = None, operators = None, **kw):
        """
        Search for subobjects based on parameter and operators. ::
        
            parameter = dict. see nive.Search
            operators = dict. see nive.Search
            kw.sort = sort objects. if None container default sort is used
            kw.batch = load subobjects as batch
            **kw = see Container.GetObj()
            returns all matching subobjects as list
    
        see :class:`nive.Search` for parameter/operators description
        
        Events
        - loadObj(obj)
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        parameter[u"pool_unitref"] = self.id
        sort = kw.get("sort", self.GetSort())
        fields = [u"id",u"pool_datatbl",u"pool_dataref",u"pool_type",u"pool_state"]
        root = self.root()
        if kw.get("queryRestraints")!=False:
            parameter,operators = root.ObjQueryRestraints(self, parameter, operators)
        objects = root.SelectDict(parameter=parameter, fields=fields, operators=operators, sort = sort)
        useBatch = kw.get("batch", True)
        if useBatch:
            ids = [c["id"] for c in objects]
            kw["meta"] = objects
            kw["sort"] = sort
            objs = self._GetObjBatch(ids, queryRestraint=False, **kw)
            return objs
        objs = []
        for c in objects:
            try:
                kw[u"pool_datatbl"] = c[u"pool_datatbl"]
                kw[u"pool_dataref"] = c[u"pool_dataref"]
                obj = self._GetObj(c[u"id"], queryRestraints=False, **kw)
                if obj:
                    objs.append(obj)
            except:
                pass
        self.Signal("loadObj", objs)
        return objs


    def GetObjsList(self, fields = None, parameter = None, operators = None, pool_type = None, **kw):
        """
        Search for subobjects based on parameter and operators. This function performs a sql query based on parameters and
        does not load any object. ::
        
            fields = list. see nive.Search
            parameter = dict. see nive.Search
            operators = dict. see nive.Search
            kw.sort = sort objects. if None container default sort is used
            kw.batch = load subobjects as batch
            returns dictionary list
    
        see :class:`nive.Search` for parameter/operators description
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        parameter[u"pool_unitref"] = self.id
        sort = kw.get("sort", self.GetSort())
        if not fields:
            fields = [u"id", u"title", u"pool_filename", u"pool_type", u"pool_state", u"pool_sort", u"pool_stag", u"pool_wfa"]
        root = self.root()
        parameter,operators = root.ObjQueryRestraints(self, parameter, operators)
        if pool_type:
            objects = root.SelectDict(pool_type=pool_type, parameter=parameter, fields=fields, operators=operators, sort = sort)
        else:
            objects = root.SelectDict(parameter=parameter, fields=fields, operators=operators, sort = sort)
        return objects
    
        
    def GetObjsBatch(self, ids, **kw):
        """
        Tries to load the objects with as few sql queries as possible. ::
        
            ids = list of object ids as number
            **kw = see Container.GetObj()
            returns all matching sub objects as list

        Events
        - loadObj(objs)
        """
        sort = kw.get("sort", self.GetSort())
        fields = [u"id",u"pool_datatbl",u"pool_dataref"]
        parameter,operators = {}, {}
        parameter[u"id"] = ids
        parameter[u"pool_unitref"] = self.id
        operators[u"id"] = "IN"
        root = self.root()
        parameter,operators = root.ObjQueryRestraints(self, parameter, operators)
        objects = root.SelectDict(parameter=parameter, fields=fields, operators=operators, sort = sort)
        ids = [c["id"] for c in objects]
        kw["meta"] = objects
        objs = self._GetObjBatch(ids, **kw)
        self.Signal("loadObj", objs)
        return objs
    
        
    def GetContainers(self, parameter = None, operators = None, **kw):
        """
        Loads all subobjects with container functionality. Uses select tag range `nive.definitions.StagContainer` to 
        `nive.definitions.StagPageElement - 1`.  ::
        
            kw.batch = load subobjects as batch and not as single object
            parameter = see pool Search
            operators = see pool Search
            **kw = see Container.GetObj()
            returns all matching sub objects as list

        see :class:`nive.Search` for parameter/operators description

        Events
        - loadObj(objs)
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        parameter[u"pool_stag"] = (StagContainer, StagPageElement-1)
        operators[u"pool_stag"] = u"BETWEEN"
        objs = self.GetObjs(parameter, operators, **kw)        
        return objs

        
    def GetContainerList(self, fields = None, parameter = None, operators = None, **kw):
        """
        Lists all subobjects with container functionality. Uses select tag range `nive.definitions.StagContainer` to 
        `nive.definitions.StagPageElement - 1`. This function performs a sql query based on parameters and
        does not load any object. ::

            fields = list. see nive.Search
            parameter = see pool Search
            operators = see pool Search
            kw.batch = load subobjects as batch
            **kw = see Container.GetObj()
            returns dictionary list

        see :class:`nive.Search` for parameter/operators description
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        parameter[u"pool_stag"] = (StagContainer, StagPageElement-1)
        operators[u"pool_stag"] = u"BETWEEN"
        return self.GetObjsList(fields, parameter, operators, **kw)
    
        
    def GetContainedIDs(self, sort=None):
        """
        Returns the ids of all contained objects including subfolders sorted by *sort*. Default is *GetSort()*.

        returns list
        """
        if not sort:
            sort = self.GetSort()
        db = self.app.db
        ids = db.GetContainedIDs(sort=sort, base=self.id)
        return ids


    def GetSort(self):
        """
        The default sort order field name.
        
        returns the field id as string
        """
        return u"title"


    def IsContainer(self):
        """ """
        return IContainer.providedBy(self)


class ContainerEdit:
    """
    Container with add and delete functionality for subobjects.

    Requires: Container
    """

    def Create(self, type, data, user, **kw):
        """
        Creates a new sub object. ::
        
            type = object type id as string or object configuration
            data = dictionary containing data for the new object
            user = the currently active user
            **kw = version information
            returns the new object or None
            
        Events
        
        - beforeAdd(data=data, type=type, user=user, kw) called for the container
        - create(user=user, kw) called for the new object
        - afterAdd(obj=obj, user=user, kw) called for the container after object has been committed
        
        Workflow actions
        
        - add (called in context of the container)
        - create (called in context of the new object)
        """
        app = self.GetApp()
        typedef = app.GetObjectConf(type)
        if not typedef:
            raise ConfigurationError, "Type not found (%s)" % (str(type))

        # allow subobject
        if not self.IsTypeAllowed(type, user):
            raise ContainmentError, "Add type not allowed here (%s)" % (str(type))

        if not self.WfAllow("add", user=user):
            raise WorkflowNotAllowed, "Not allowed in current workflow state (add)"

        self.Signal("beforeAdd", data=data, type=type, user=user, **kw)
        db = app.db
        dbEntry = None
        try:
            dbEntry = db.CreateEntry(pool_datatbl=typedef["dbparam"], user=user, **kw)
            obj = self._GetObj(dbEntry.GetID(), dbEntry = dbEntry, parentObj = self, configuration = typedef, **kw)
            obj.CreateSelf(data, user=user, **kw)
            self.WfAction("add", user=user)
            obj.Signal("create", user=user, **kw)
            if app.configuration.autocommit:
                obj.CommitInternal(user=user)
        except Exception, e:
            #try:
            #    if dbEntry:
            #        id = dbEntry.GetID()
            #        db.DeleteEntry(id)
            #except:
            #    pass
            db.Undo()
            raise 
        self.Signal("afterAdd", obj=obj, user=user, **kw)
        return obj


    def Duplicate(self, obj, user, updateValues=None, **kw):
        """
        Duplicate the object including all data and files and store as new subobject. ::
        
            obj = the object to be duplicated
            user = the currently active user
            updateValues = dictionary containing meta, data, files
            **kw = version information
            returns new object or None
            
        Events
        
        - beforeAdd(data=data, type=type, user=user, kw) called for the container
        - duplicate(kw) called for the new object
        - afterAdd(obj=obj, user=user, kw) called for the container after obj has been committed

        Workflow action

        - add (called in context of the container)
        - create (called in context of the new object)
        """
        if not updateValues:
            updateValues={}
        app = self.GetApp()
        type=obj.GetTypeID()
        # allow subobject
        if not self.IsTypeAllowed(type, user):
            raise ContainmentError, "Add type not allowed here (%s)" % (str(type))
        
        if not self.WfAllow("add", user=user):
            raise WorkflowNotAllowed, "Workflow: Not allowed (add)"

        self.Signal("beforeAdd", data=updateValues, type=type, user=user, **kw)
        newDataEntry = None
        try:
            newDataEntry = obj.dbEntry.Duplicate()
            updateValues["pool_unitref"] = self.GetID()
            updateValues["pool_wfa"] = ""
            data, meta, files = obj.SplitData(updateValues)
            newDataEntry.meta.update(meta)
            newDataEntry.data.update(data)
            newDataEntry.files.update(files)
            id = newDataEntry.GetID()
            typedef = obj.configuration
    
            newobj = self._GetObj(id, dbEntry = newDataEntry, parentObj = self, configuration = typedef)
            newobj.CreateSelf(data, user=user)
            newobj.Signal("duplicate", **kw)
            
        except Exception, e:
            try:
                if newDataEntry:
                    id = newDataEntry.GetID()
                    db = app.db
                    db.DeleteEntry(id)
            except:
                pass
            db.Undo()
            raise 

        try:    
            if obj.IsContainer():
                for o in obj.GetObjs(queryRestraints=False):
                    newobj._RecursiveDuplicate(o, user, **kw)
            self.WfAction("add", user=user)
            if app.configuration.autocommit:
                newobj.CommitInternal(user=user)
        except Exception, e:
             self._DeleteObj(newobj)
             raise 
         
        self.Signal("afterAdd", obj=newobj, user=user, **kw)
        return newobj
    
        
    def _RecursiveDuplicate(self, obj, user, **kw):
        """
        Recursively duplicate all subobjects. 
        """
        type=obj.GetTypeID()
        
        updateValues = {}
        self.Signal("beforeAdd", data=updateValues, type=type, **kw)
        newDataEntry = obj.dbEntry.Duplicate()
        updateValues["pool_unitref"] = self.GetID()
        updateValues["pool_wfa"] = ""
        data, meta, files = obj.SplitData(updateValues)
        newDataEntry.meta.update(meta)
        newDataEntry.data.update(data)
        newDataEntry.files.update(files)
        id = newDataEntry.GetID()
        typedef = obj.configuration

        newobj = self._GetObj(id, dbEntry = newDataEntry, parentObj = self, configuration = typedef)
        newobj.CreateSelf({}, user=user)
        newobj.Signal("duplicate", **kw)
        
        if obj.IsContainer():
            for o in obj.GetObjs(queryRestraints=False):
                newobj._RecursiveDuplicate(o, user, **kw)

        if self.app.configuration.autocommit:
            newobj.CommitInternal(user=user)

        self.Signal("afterAdd", obj=newobj, **kw)


    def Delete(self, id, user, obj=None, **kw):
        """
        Delete the subobject referenced by id. ::
        
            id = id of object to be deleted
            user = the currently active user
            obj = the object to be deleted. Will be loaded automatically if None
            **kw = version information
            returns True or False
        
        Events
        
        - delete(user=user) called on object to be deleted
        - afterDelete(id=id, user=user) called on container after object has been deleted

        Workflow action

        - remove (called in context of the container)
        - delete (called in context of the new object)
        """
        app = self.GetApp()
        if not obj:
            obj = self.GetObj(id, queryRestraints=False, **kw)
            if not obj:
                return False
        if obj.GetParent().id!=self.id:
            raise ContainmentError, "Object is not a child (%s)" % (str(id))

        # call workflow
        if not self.WfAllow("remove", user=user):
            raise WorkflowNotAllowed, "Workflow: Not allowed (remove)"
        if not obj.WfAllow("delete", user=user):
            raise WorkflowNotAllowed, "Workflow: Not allowed (delete)"
        obj.Signal("delete", user=user)
        if hasattr(obj, "_RecursiveDelete"):
            obj._RecursiveDelete(user)

        # call workflow
        obj.WfAction("delete", user=user)
        self._DeleteObj(obj)
        if app.configuration.autocommit:
            self.db.Commit()
        self.WfAction("remove", user=user)
        self.Signal("afterDelete", id=id, user=user)

        return True


    def DeleteInternal(self, id, user, obj=None, **kw):
        """
        Like *Delete()* but does not call any workflow action. ::
        
            id = id of object to be deleted
            user = the currently active user
            obj = the object to be deleted. Will be loaded automatically if None
            **kw = version information
            returns True or False

        Events
        
        - delete() called on object to be deleted
        - afterDelete(id=id) called on container after object has been deleted
        """
        app = self.GetApp()
        if not obj:
            obj = self.GetObj(id, queryRestraints=False, **kw)
        else:
            if not obj.GetParent()==self:
                raise ContainmentError, "Object is not a child (%s)" % (str(id))
        obj.Signal("delete")
        if hasattr(obj, "_RecursiveDeleteInternal"):
            obj._RecursiveDeleteInternal(user)
        self._DeleteObj(obj)
        self.Signal("afterDelete", id=id)
        return 1


    def _RecursiveDelete(self, user):
        """
        Recursively deletes all subobjects
        """
        objs = self.GetObjs(queryRestraints=False)
        for o in objs:
            self.Delete(o.id, obj=o, user=user)


    def _RecursiveDeleteInternal(self, user):
        """
        Recursively deletes all subobjects
        """
        objs = self.GetObjs(queryRestraints=False)
        for o in objs:
            self.DeleteInternal(o.id, obj=o, user=user)
            
            
class ContainerSecurity:
    """
    Provides functionality to allow or disallow the creation of objects based
    on object configuration.subtypes.
    """

    def GetAllowedTypes(self, user=None, visible=1):
        """
        List types allowed to be created in this container based on
        configuration.subtypes.    ::
        
            user = the currently active user
            visible = will skip all hidden object types 
            returns list of configurations
            
        """
        subtypes = self.configuration.subtypes
        if not subtypes:
            return False
        all = self.app.GetAllObjectConfs(visibleOnly=visible)
        if subtypes == "*":
            return all

        # check types by interface
        allowed = []
        for conf in all:
            # create type from configuration
            type = self._GetVirtualObj(conf)
            if not type:
                continue
            # loop subtypes
            for iface in subtypes:
                if isinstance(iface, basestring):
                    iface = ResolveName(iface, raiseExcp=False)
                    if not iface:
                        continue
                try:
                    if iface.providedBy(type):
                        allowed.append(conf)
                except:
                    pass
        return allowed


    def IsTypeAllowed(self, type, user=None):
        """
        Check if *type* is allowed to be created in this container based on
        configuration.subtypes.::
        
            type = the type to be checked
            user = the currently active user
            returns True/False
        
        *type* can be passed as
        
        - type id string
        - configuration object
        - type object instance
        """
        if not type:
            return False
        subtypes = self.configuration.subtypes
        if subtypes == "*":
            return True
        if not subtypes:
            return False
        if isinstance(type, basestring):
            # dotted python to obj configuration
            type = self.app.GetObjectConf(type)
            if not type:
                return False
            # create type from configuration
            type = self._GetVirtualObj(type)
            if not type:
                return False
        if not IObject.providedBy(type) and not IConf.providedBy(type):
            return False
        # loop subtypes
        for iface in subtypes:
            if isinstance(iface, basestring):
                iface = ResolveName(iface, raiseExcp=False)
                if not iface:
                    continue
            try:
                if iface.providedBy(type):
                    return True
            except:
                pass
        return False
    

class ContainerFactory:
    """
    Container object factory. Creates objects based on type configuration.
    """

    def _GetObj(self, id, dbEntry = None, parentObj = None, configuration = None, **kw):
        """
        Loads and initializes the object. ::
        
            id = id of the object to be loaded
            dbEntry = the database entry. Will be loaded automatically if None
            parentObj = if a different parent than the container
            configuration = the object configuration to be loaded
            returns the object or None
            
        """
        useCache = ICache.providedBy(self)
        if useCache:
            o = self.GetFromCache(id)
            if o:
                return o
        app = self.app
        if not dbEntry:
            # check restraints
            qr = kw.get("queryRestraints",None)
            if qr!=False:
                root = self.root()
                p,o = root.ObjQueryRestraints(self)
                p["id"] = id
                p["pool_unitref"] = self.id
                e = root.Select(parameter=p, operators=o)
                if len(e)==0:
                    return None

            dbEntry = app.db.GetEntry(id, **kw)
            if not dbEntry:
                return None
                #raise Exception, "NotFound"

            if dbEntry.meta["pool_unitref"]!=self.id:
                raise ContainmentError, "Object is not a child (%s)" % (str(id))

        # create object for type
        if not parentObj:
            parentObj = self
        obj = None
        if not configuration:
            type = dbEntry.GetMetaField("pool_type")
            if not type:
                # broken entry 
                #raise ConfigurationError, "Empty type"
                return None
            configuration = app.GetObjectConf(type)
            if not configuration:
                raise ConfigurationError, "Type not found (%s)" % (str(type))
        obj = ClassFactory(configuration, app.reloadExtensions, True, base=None)
        obj = obj(id, dbEntry, parent=parentObj, configuration=configuration, **kw)
        if useCache:
            self.Cache(obj, obj.id)
        return obj


    def _GetVirtualObj(self, configuration):
        """
        This loads an object for a non existing database entry.
        """
        if not configuration:
            raise ConfigurationError, "Type not found"
        app = self.app
        obj = ClassFactory(configuration, app.reloadExtensions, True, base=None)
        dbEntry = app.db.GetEntry(0, virtual=1)
        obj = obj(0, dbEntry, parent=None, configuration=configuration)
        return obj

    
    def _GetObjBatch(self, ids, parentObj = None, **kw):
        """
        Load multiple objects at once.
        """
        if len(ids)==0:
            return []
        useCache = ICache.providedBy(self)
        objs = []
        if useCache:
            load = []
            for id in ids:
                o = self.GetFromCache(id)
                if o:
                    objs.append(o)
                else:
                    load.append(id)
            if len(load)==0:
                return objs
            ids = load

        app = self.GetApp()
        entries = app.db.GetBatch(ids, preload="all", **kw)

        # create object for type
        if not parentObj:
            parentObj = self
        for dbEntry in entries:
            obj = None
            type = dbEntry.meta.get("pool_type")
            if not type:
                continue
            configuration = app.GetObjectConf(type, skipRoot=1)
            if not configuration:
                continue
            obj = ClassFactory(configuration, app.reloadExtensions, True, base=None)
            obj = obj(dbEntry.id, dbEntry, parent=parentObj, configuration=configuration, **kw)
            if useCache:
                self.Cache(obj, obj.id)
            objs.append(obj)
        return objs


    def _DeleteObj(self, obj, id=0):
        """
        Deletes the object and additional data from wfdata, wflog, fulltext,
        security tables.
        """
        if obj:
            id = obj.GetID()
        obj.Close()
        db = self.db
        if id > 0:
            db.DeleteEntry(id)



class Root(object):
    """
    The root is a container for objects but does not store any data in the database itself. It
    is the entry point for object access. Roots are only handled by the application. 

    Requires (Container, ContainerFactory, Event)
    """
    
    def __init__(self, path, app, rootDef):
        self.__name__ = str(path)
        self.__parent__ = app
        self.id = 0 
        # unique root id generated from name . negative integer.
        self.idhash = abs(hash(self.__name__))*-1
        self.path = path
        self.configuration = rootDef
        self.queryRestraints = {}, {}

        self.meta = Conf(pool_type=rootDef["id"], 
                         title=rootDef["name"], 
                         pool_state=1, 
                         pool_filename=path, 
                         pool_wfa=u"",
                         pool_change=datetime.now()) # set change time to now by default
        self.data = Conf()
        self.Signal("init")


    # Properties -----------------------------------------------------------

    def root(self):
        """ this will return itself. Used for object compatibility. """
        return self

    @property
    def app(self):
        """ returns the cms application the root is used for """
        return self.__parent__

    @property
    def db(self):
        """ returns the datapool object """
        return self.app.db

    @property
    def parent(self):
        """ this will return None. Used for object compatibility. """
        return None

    # Object Lookup -----------------------------------------------------------

    def LookupObj(self, id, **kw):
        """
        Lookup the object referenced by id *anywhere* in the tree structure. Use obj() to 
        restrain lookup to the first sublevel only. ::
        
            id = number
            **kw = version information
        
        returns the object or None
        """
        try:
            id = long(id)
        except:
            return None
        if id <= 0:
            return self
        if not id:
            return None
            #raise Exception, "NotFound"

        # proxy object
        if kw.has_key("proxyObj") and kw["proxyObj"]:
            obj = obj._GetObj(id, parentObj = kw["proxyObj"], **kw)
            if not obj:
                raise ContainmentError, "Proxy object not found"
            return obj

        # load tree structure
        path = self.app.db.GetParentPath(id)
        if path == None:
            return None
            #raise Exception, "NotFound"
        
        # check and adjust root id
        if hasattr(self, "rootID"):
            if self.rootID in path:
                path = path[path.index(self.rootID)+1:]
        if hasattr(self.app, "rootID"):
            if self.app.rootID in path:
                path = path[path.index(self.app.rootID)+1:]

        # reverse lookup of object tree. loads all parent objs.
        path.append(id)
        #opt
        obj = self
        for id in path:
            if id == self.id:
                continue
            obj = obj._GetObj(id, **kw)
            if not obj:
                return None
                #raise Exception, "NotFound"
        return obj


    def LookupTitle(self, id):
        """
        Lookup the object title referenced by id *anywhere* in the tree structure.
        
        returns title as string or empty string
        """
        p, o = self.ObjQueryRestraints(self)
        p["id"] = id
        v = self.Select(parameter = p, operators = o, fields = [u"title"], start = 0, max = 1)
        if len(v):
            return v[0][0]
        return ""


    def ObjQueryRestraints(self, containerObj=None, parameter=None, operators=None):
        """
        The functions returns two dictionaries (parameter, operators) used to restraint
        object lookup in subtree. For example a restraint can be set to ignore all
        objects with meta.pool_state=0. All container get (GetObj, GetObjs, ...) functions 
        use query restraints internally. 
        
        See `nive.search` for parameter and operator usage.
        
        Please note: Setting the wrong values for query restraints can easily crash 
        the application. 
        
        Event:
        - loadRestraints(parameter, operators)
        
        returns parameter dict, operators dict
        """
        p, o = self.queryRestraints
        if parameter:
            parameter.update(p)
            if operators:
                operators.update(o)
            else:
                operators = o.copy()
        else:
            parameter=p.copy()
            operators=o.copy()
        self.Signal("loadRestraints", parameter=parameter, operators=operators)
        return parameter, operators


    # Values ------------------------------------------------------

    def GetID(self):
        """ returns 0. the root id is always zero. """
        return self.id

    def GetTypeID(self):
        """ returns the root type id from configuration """
        return self.configuration.id

    def GetTypeName(self):
        """ returns the root type name from configuration """
        return self.configuration.name

    def GetTitle(self):
        """ returns the root title from configuration. """
        return self.meta["title"]

    def GetPath(self):
        """ returns the url path name as string. """
        return self.__name__


    # Parents ----------------------------------------------------

    def IsRoot(self):
        """ returns always True. """
        return True

    def GetRoot(self):
        """ returns self. """
        return self

    def GetApp(self):
        """ returns the cms application. """
        return self.app

    def GetParent(self):
        """ returns None. """
        return None

    def GetParents(self):
        """ returns empty list. Used for object compatibility. """
        return []

    def GetParentIDs(self):
        """ returns empty list. Used for object compatibility. """
        return []

    def GetParentTitles(self):
        """ returns empty list. Used for object compatibility. """
        return []

    def GetParentPaths(self):
        """ returns empty list. Used for object compatibility. """
        return []


    # tools ----------------------------------------------------

    def GetTool(self, name):
        """
        Load a tool in the roots' context. Only works for tools registered for roots or this root type. ::
            
            returns the tool object or None
        
        Event
        - loadToool(tool=toolObj)
        """
        t = self.app.GetTool(name, self)
        self.Signal("loadTool", tool=t)
        return t


    def Close(self):
        """
        Close the root and all contained objects. Currently only used in combination with caches.
        
        Event
        - close()
        """
        self.Signal("close")
        if ICache.providedBy(self):
            #opt
            for o in self.GetAllFromCache():
                o.Close()
        return


class RootWorkflow:
    """
    Provides workflow functionality for roots. 
    Workflow process objects are handled and loaded by the application. Meta layer fields `pool_wfp` stores the 
    process id and `pool_wfa` the current state.
    """

    def WfAllow(self, action, user, transition = None):
        """
        Check if action is allowed in current state in workflow. This functions returns True or False
        and unlike WFAction() will not raise a WorkflowNotAllowed Exception.
        
        Event: 
        - wfAllow(name)

        returns bool
        """
        wf = self.GetWf()
        if not wf:
            return True
        self.Signal("wfAllow", name=action)
        return wf.Allow(action, self, user=user, transition=transition)


    def WfAction(self, action, user, transition = None):
        """
        Trigger workflow action. If several transitions are possible for the action in the current state
        the first is used. In this case the transition id can be passed as parameter.
        
        Event: 
        - wfAction(name)

        raises WorkflowNotAllowed 
        """
        wf = self.GetWf()
        if not wf:
            return 
        wf.Action(action, self, user=user, transition=transition)
        self.Signal("wfAction", name=action)


    def GetWf(self):
        """
        Returns the workflow process for the object.

        Event: 
        - wfLoad(workflow) 
        
        returns object
        """
        app = self.app
        if not app.configuration.workflowEnabled:
            return None
        wfTag = self.GetNewWf()
        if not wfTag:
            return None
        # load workflow
        wf = app.GetWorkflow(wfTag, contextObject=self)
        # enable strict workflow checking
        if not wf:
            raise ConfigurationError, "Workflow process not found (%s)"%(wfTag)
        self.Signal("wfLoad", workflow=wf)
        return wf


    def GetNewWf(self):
        """
        Returns the workflow process id for the object based on configuration settings. 

        returns string
        """
        if self.configuration.workflowDisabled:
            return ""
        return self.configuration.workflowID


    def GetWfInfo(self, user):
        """
        returns the current workflow state as dictionary
        """
        wf = self.GetWf()
        if not wf:
            return {}
        return wf.GetObjInfo(self, user)


    def GetWfState(self):
        """
        """
        return self.meta["pool_wfa"]


    def SetWfState(self, stateID):
        """
        Sets the workflow state for the object. The new is state is set
        regardless of transitions or calling any workflow actions.
        """
        self.meta["pool_wfa"] = stateID

    




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Using configuration classes
-----------------------------
The predefined configuration classes are meant to handle the different cms 
modules and parts. Empty configurations are already filled with default values for convenience.
Also configuration classes provide functions for easy copying and changing. 

You have to use one of these to extend or change anything in the application.

Each configuration class can be used with python dictionary or attribute notation to set
or get values.    
    
Values can be defined in python files as ::

    baseConf(id='test',name='test')

or ::

    c = baseConf()
    c.id = 'test'
    c.name = 'test'

or dictionary syntax: ::

    baseConf(**{'id':'test','name':'test'})
    c["id"]
    c.get("id")
    c.update(values)
    c.keys()
    c.copy()

Alternatively configurations can be defined outside python code as json files. The specific
configuration class is mapped from the type value ::

    {
        "type":    "ObjectConf",
        "copyFrom":"nive.cms.app",
        "id":      "my_object",
        "name":    "My Object",
        "dbparam": "my_object",
        "context": "website.my_object",
        ...
    }

Json files are distinguished by their file extension ".json" from python dotted path names. You
can manually load json files by calling nive.helper.LoadConfiguration(filename). ::

    nive.helper.LoadConfiguration("my_object.json")

To copy and customize existing configurations call the constructor with the 
configuration to be copied as first non-keyword parameter: ::

    a = baseConf(id='test',name='test')
    b = baseConf(a, id='test2')

or ::

    cms = AppConf("nive.cms.app", id="my_website", ...)

Json: add "copyFrom" ::

    {
        "type":    "AppConf",
        "copyFrom":"nive.cms.app",
        "id":      "my_website",
        ...
    }    

Data field types
----------------
These types define both: the data storage type (like string or number), widget
and internal handling. Use these for FieldConf.datatypes.

===========  ==============================================
id           Name
===========  ==============================================
string       String 
number       Number 
float        Float 
bool         Bool 
htext        HTML Text 
text         Text 
code         HTML / Javascript / CSS 
json         Json 
file         File 
date         Date 
datetime     Datetime 
list         List 
radio        Radio Selection 
mselection   Multiple Selection 
mcheckboxes  Multiple Checkboxes 
lines        Lines 
email        E-Mail 
url          URL 
urllist      URL List 
password     Password 
unit         ID Reference 
unitlist     ID Reference List 
timestamp    Timestamp 
===========  ==============================================


Configurations
---------------
"""

import copy
from types import ListType,TupleType,DictType

from zope.interface import Interface, Attribute, implements, Provides, alsoProvides, directlyProvidedBy, directlyProvides

from pyramid.path import DottedNameResolver
from pyramid.renderers import render

from nive.i18n import _


# interfaces ----------------------------------------------------------------------
# objects, root and application
class IApplication(Interface):
    pass
class IRoot(Interface):
    pass
class IObject(Interface):
    pass
class IContainer(Interface):
    pass
class INonContainer(Interface):
    pass
class IReadonly(Interface):
    pass
class IPortal(Interface):
    pass

# pages and elements
class IWebsite(Interface):
    pass
class IPage(Interface):
    pass
class IPageContainer(Interface):
    pass
class IPageElement(Interface):
    pass
class IPageElementContainer(Interface):
    pass
class IFolder(Interface):
    pass
class IColumn(Interface):
    pass
class IFile(Interface):
    pass
class ITool(Interface):
    pass
class IIFaceRoot(Interface):
    pass
class ICMSRoot(Interface):
    pass
class IWebsiteRoot(Interface):
    pass
class ICache(Interface):
    pass
class IPersistent(Interface):
    pass
class IUser(Interface):
    pass
        
# configuration 
class IConf(Interface):
    pass
class IAppConf(Interface):
    pass
class IDatabaseConf(Interface):
    pass
class IRootConf(Interface):
    pass
class IObjectConf(Interface):
    pass
class IToolConf(Interface):
    pass
class IViewModuleConf(Interface):
    pass
class IViewConf(Interface):
    pass
class IModuleConf(Interface):
    pass
class IWidgetConf(Interface):
    pass
class IPortalConf(Interface):
    pass

class IFieldConf(Interface):
    pass
class IGroupConf(Interface):
    pass
class IAclConf(Interface):
    pass
class ICategoryConf(Interface):
    pass
class IWorkflowConf(Interface):
    pass

class IFileStorage(Interface):
    """
    Used for file storage classes on object level.
    """
    pass
    
# workflow
class IWfProcessConf(Interface):
    pass
class IWfStateConf(Interface):
    pass
class IWfTransitionConf(Interface):
    pass
class IProcess(Interface):
    pass


# userdb
class IUserDatabase(Interface):
    pass
class ILocalGroups(Interface):
    pass

# interface extensions
class IToolboxWidgetConf(Interface):
    """
    IToolboxWidgetConf refers to the nive.cmsview toolbox plugin point. Use IToolboxWidgetConf as widgetType
    in your WidgetConf() to link a widget to the nive toolbox.
    """

class IEditorWidgetConf(Interface):
    """
    IEditorWidgetConf refers to the nive.cmsview editor tab plugin point. Use IEditorWidgetConf as widgetType
    in your WidgetConf() to link a new tab to the nive editor header.
    """

class IAdminWidgetConf(Interface):
    """
    IAdminWidgetConf refers to the nive.adminview tab plugin point. Use IAdminWidgetConf as widgetType
    in your WidgetConf() to link a new tab to the nive admin header.
    """

class ISort(Interface):
    pass




class baseConf(object):
    locked=0
    
    def copy(self, **newvalues):
        data = copy.deepcopy(self.__dict__)
        data.update(newvalues)
        data["locked"] = 0
        return self.__class__(**data)
    
    def test(self):
        return True
    
    #Provides(IConf)
    def __init__(self, copyFrom=None, **values):
        self._empty = True
        self._parent = None
        #alsoProvides(self, IConf)
        if copyFrom:
            self._empty=False
            import helper
            i,c = helper.ResolveConfiguration(copyFrom)
            if c:
                data = copy.deepcopy(c.__dict__)
                self.update(data)
                self._parent = c
        if values:
            self._empty=False
            self.update(values)
            
    def __len__(self):
        return not self._empty
    
    def keys(self):
        k = self.__dict__.keys()
        k.remove("_empty")
        k.remove("_parent")
        try:
            k.remove("locked")
        except:
            pass
        return k
    
    def has_key(self, key):
        if hasattr(self, key):
            return True
        return False
    
    def update(self, values):
        self._empty=False
        #opt
        for k in values.keys():
            setattr(self, k, values[k])
            
    @property
    def parent(self):
        return self._parent
            
    def get(self, key, default=None):
        if key in self.__dict__:
            return self.__dict__[key]
        return default

    def __getattr__(self, key):
        if key in self.__dict__:
            return self.__dict__[key]
        raise AttributeError, key

    def __setattr__(self, key, value):
        if self.locked:
            raise ConfigurationError, "Configuration locked."
        self.__dict__["_empty"]=False
        self.__dict__[key]=value

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, value):
        if self.locked:
            raise ConfigurationError, "Configuration locked."
        self._empty=False
        setattr(self, key, value)
        
    def __iter__(self):
        return iter(self.keys())

    def __deepcopy__(self, memo):
        data = copy.deepcopy(self.__dict__)
        if "locked" in data:
            del data["locked"]
        if "_empty" in data:
            del data["_empty"]
        return self.__class__(**data)
    
    def __str__(self):
        if hasattr(self, "id"):
            return str(self.id)
        elif hasattr(self, "name"):
            return str(self.name)
        return ""

    def __repr__(self):
        return "<%s at 0x%x (%s)>" % (self.__class__.__name__, abs(id(self)), str(self))

    def uid(self):
        # create a unique identifier for the configuration to support persistence
        return "nive."+self.__class__.__name__+"."+self.id
    
    def lock(self):
        self.__dict__["locked"] = 1

    def unlock(self):
        self.__dict__["locked"] = 0
    

class FieldConf(baseConf):
    """
    Definition of a field used for object.data, object.meta, form.field and tool.data

    Values ::

        *id :       Unique field id as ascii string with maximum 25 chars. Used as column name
                    in database tables.
        *datatype : Datatype of the field based on nive.definitions.DataTypes.
        size :      Field size of stored data. Depends on datatype. 
        default :   Default value.
        listItems : Used for list and checkbox types. Contains items with id and name 
                    definitions [{"id": "id1", "name": "name1"},...].
                    If listItems is a callable the callback must take two arguments:
                    callback(fieldconf, object) and return a list with id and name items.
        settings :  Extended settings for fields. Possible values depend on datatype.
        fulltext :  Use this field in fulltext index.
        
    Extended values (optional and used as default for forms) ::

        name :        Display name. 
        description : Description.
        required :    Form input is required.
        readonly :    This field is not included in forms.
        hidden :      This field is rendered as hidden field in forms.
        len :         Depends on datatype. 
        node :        Form field object for rendering and validation (colander schema node).
    
    Call ``FieldConf().test()`` to verify configuration values.

    Interface: IFieldConf
    """
    implements(IFieldConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.datatype = ""
        self.size = 0
        self.default = u""
        self.listItems = None
        self.settings = {}
        self.fulltext = False
        # used as default for forms
        self.name = u""
        self.description = u""
        self.required = False
        self.readonly = False
        self.hidden = False
        self.len = 50
        self.node = None
        baseConf.__init__(self, copyFrom, **values)

    def __str__(self):
        return "%(id)s (%(name)s): %(datatype)s" % self

    def test(self):
        report = []
        # check id
        if len(self.id) > 25:
            report.append((ConfigurationError, " FieldConf.id too long. 25 chars allowed.", self))
        # check datatype
        ok=0
        for d in DataTypes:
            if d["id"] == self.datatype:
                ok=1
                break
        if not ok:
            report.append((ConfigurationError, " FieldConf.datatype unknown.", self))
        return report


class AppConf(baseConf):
    """
    Application configuration class. Contains all configurable options for the cms 
    (nive.Application).
    
    Values ::

        *id :          Ascii used as name in url traversal.
        *context :     Dotted python name or class reference used as factory.
        title :        Application title (optional).
        description :  Application description (optional).
            
        # extensions
        meta :    Additional meta layer fields or replacements for system fields.
        modules : List of included configurations. Calls nive.Registration.Include for 
                  each module on startup. 
                  Refer to helper.ResolveConfiguration for possible values. 
        categories : list of CategoryConf definitions.
    
        # security
        groups : List of Definitions.GroupConf with groups used in this application.
        acl :    List of pyramid acls for permission settings.
            
        # options
        fulltextIndex :    Enable fulltext index based on FieldConf.fulltext setting.
        autocommit :       Enable autocommit for object write and delete operations.
        useCache :         Cache database on application level.
        frontendCodepage : Default=utf-8. The codepage used to render the html frontend.
        workflowEnabled :  Enable or disable the workflow engine.
        events  : Register for one or multiple Application events. 
                  Register each event as e.g. Conf(event="run", callback=function).
        
    Call ``AppConf().test()`` to verify configuration values.
    
    Interface: IAppConf
    """
    implements(IAppConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = "app"
        self.context = "nive.components.objects.base.ApplicationBase"
        
        self.title = u""
        self.description = u""
        
        # data pool
        self.autocommit = True
        self.useCache = True
        self.frontendCodepage = "utf-8"
        self.fulltextIndex = False
        self.workflowEnabled = False
        
        # security
        self.groups = []
        self.acl = []
        
        # extensions
        self.meta = []
        self.modules = []
        self.categories = []
        
        # development
        self.reloadExtensions = False
        baseConf.__init__(self, copyFrom, **values)

        # bw 0.9.3 
        # database connection parameter
        if hasattr(self, "dbConfiguration") and isinstance(self.dbConfiguration, basestring):
            import helper
            self.dbConfiguration = helper.LoadConfiguration(self.dbConfiguration)
        
 
    def __str__(self):
        return "%(id)s (%(title)s) implemented by %(context)s" % self

    def uid(self):
        # create a unique identifier for the configuration to support persistence
        return "nive."+self.__class__.__name__

    
    def test(self):
        report = []
        #check id
        if not self.id:
            report.append((ConfigurationError, " AppConf.id is empty", self))
        # check context
        o = TryResolveName(self.context)
        if not o:
            report.append((ImportError, " for AppConf.context", self))
        # check modules
        for m in self.modules:
            if isinstance(m, basestring):
                o = TryResolveName(m)
                if not o:
                    report.append((ImportError, " AppConf.modules error: "+m, self, m))
            if hasattr(m, "test"):
                report += m.test()
        # check meta
        if not type(self.meta) in (ListType,TupleType):
            report.append((TypeError, " AppConf.meta error: Not a list", self))
        else:
            for m in self.meta:
                if hasattr(m, "test"):
                    report += m.test()
        return report
        

class DatabaseConf(baseConf):
    """
    Database configuration ::

        *context : Database type to be used. supports "Sqlite3" and "MySql" by default. 
                   MySql requires python-mysqldb installed.
        *dbName  : sqlite3=database file path, mysql=database name 
        fileRoot : Relative or absolute root directory for files.
        host     : database server host.
        port     : database server port.
        user     : database server user.
        password : database server password.
        useTrashcan : Move files to fileRoot.__traschcan directory on delete.
        unicode  : Database is using unicode mode.
        dbCodePage : If not in unicode mode, the database codepage used (default "utf-8").
        connection : Specifies the database connection management class. Default None.
        verifyConnection : Verify connection is still alive each time a connection is requested.
                           Automatically reconnects if the connection is closed.
        timeout  : Timeout in seconds for database requests, if supported.
        querylog : Enable database query log. "querylog" is used as filename the application 
                   will use for the query log and the number of traceback lines. 
                   use e.g. (10,'sql.log')
        events  : Register for one or multiple Application events. 
                  Register each event as e.g. Conf(event="run", callback=function).
    
    Interface: IDatabaseConf
    
    Call ``DatabaseConf().test()`` to verify configuration values.
    """
    implements(IDatabaseConf)
    
    def __init__(self, copyFrom=None, **values):
        self.context = "Sqlite3"
        self.fileRoot = ""
        self.dbName = ""
        self.host = ""
        self.port = 0
        self.user = ""
        self.password = ""
        self.useTrashcan = False
        self.unicode = True
        self.timeout = 3
        self.verifyConnection = False
        self.connection = None
        self.dbCodePage = "utf-8"
        self.querylog=(0,None)
        baseConf.__init__(self, copyFrom, **values)
        
        
    def __str__(self):
        return "%(context)s %(dbName)s / %(fileRoot)s" % self

    def uid(self):
        # create a unique identifier for the configuration to support persistence
        return "nive."+self.__class__.__name__

    def test(self):
        report = []
        # check context
        c = self.context
        if c in ("Sqlite3", "Mysql", "MySql"):
            return report
        o = TryResolveName(c)
        if not o:
            report.append((ImportError, " for DatabaseConf.context", self))
        if not self.dbName:
            report.append((ConfigurationError, " DatabaseConf.dbName is empty", self))
        return report



class ObjectConf(baseConf):
    """
    Configuration of a object type definition

    Values ::

        *id :      Unique type id as ascii string with maximum 25 chars.
        *name :    Name used for display.
        *dbparam : Database table name.
        *context : Dotted python name or class reference used as factory. 
        extensions:List of dotted python names or class references to extend context with
                   additional functionality. Used in object factory.
        data :     List of nive.definitions.FieldConf classes. Are mapped to the data table
                   except for "file" data types. 
        template : Template file name to render the default view. If file name is a
                   relative path ('text.pt') the template is looked up in the active 
                   design or view module templatePath. The file name can also be a 
                   absolute dotted python name ('nive.cms.templates:text.pt').
        views :    List of object view definitions. Either nive.definitions.ViewConf or 
                   nive.definitions.ViewModuleConf.
        forms :    Configuration of object form subsets and actions. Refer to nive.Form.
    
        subtypes  : Define possible subtypes. None=no objects allowed, "*"=all objects allowed, 
                    [IContainer,IPageElement]=list with interfaces of allowed objects.
        selectTag : Default select tag for this type. Stored as meta.pool_stag.
        category  : Default category stored as meta.pool_category.
        hidden    : Hide in select lists in user interface.
        workflowDisabled : Enable or disable workflow for this type.
        workflowID       : Workflow process id.

        events  : Register for one or multiple Application events. 
                  Register each event as e.g. Conf(event="run", callback=function).
        version   : Optional version string.
        description      : Description.

    Call ``ObjectConf().test()`` to verify configuration values.
    
    Object instances provide the configuration as `configuration` attribute.
    
    Interface: IObjectConf
    """
    implements(IObjectConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.dbparam = u""
        self.selectTag = 0
        self.context = "nive.components.objects.base.ObjectBase"
        self.template = None
        self.extensions = None
        self.data = []
        self.forms = {}
        self.views = []
        self.workflowDisabled = True
        self.workflowID = ""
        self.subtypes = "*"
        self.category = u""
        self.hidden = False
        self.events = None
        self.description = u""
        self.version = "1"
        baseConf.__init__(self, copyFrom, **values)
        import helper
        d=[]
        for field in self.data:
            if isinstance(field, dict):
                field = helper.LoadConfiguration(field)
            d.append(field)
        self.data=d


    def __str__(self):
        return "%(id)s (%(name)s) implemented by %(context)s" % self

    def test(self):
        report = []
        # check id
        if not self.id:
            report.append((ConfigurationError, " ObjectConf.id is empty", self))
        # check context
        o = TryResolveName(self.context)
        if not o:
            report.append((ImportError, " for ObjectConf.context", self))
        # check extensions
        if self.extensions:
            for e in self.extensions:
                o = TryResolveName(e)
                if not o:
                    report.append((ImportError, " for ObjectConf.extensions", e))
        # check views
        if not type(self.views) in (ListType,TupleType):
            report.append((TypeError, " ObjectConf.views error: Not a list", self))
        else:
            for m in self.views:
                if hasattr(m, "test"):
                    report += m.test()
        # check data
        if not type(self.data) in (ListType,TupleType):
            report.append((TypeError, " ObjectConf.data error: Not a list", self))
        else:
            for m in self.data:
                if hasattr(m, "test"):
                    report += m.test()
        # check forms
        if self.forms:
            for subset in self.forms:
                s = self.forms[subset]
                if not isinstance(s, dict):
                    report.append((TypeError, " ObjectConf.forms."+subset+" error: Not a dictionary", self))
                elif "fields" in s:
                    if not type(s["fields"]) in (ListType,TupleType):
                        report.append((TypeError, " ObjectConf.forms."+subset+".fields error: Not a list", self))
                elif "actions" in s:
                    if not type(s["actions"]) in (ListType,TupleType):
                        report.append((TypeError, " ObjectConf.forms."+subset+".actions error: Not a list", self))
                    
        return report



class RootConf(baseConf):
    """
    Configuration of a root object

    Values ::

        *id      : Unique type id as ascii string.
        *name    : Display name.
        *context : Dotted python name or class reference used as factory. 
        extensions:List of dotted python names or class references to extend context with
                   additional functionality. Used in object factory.
        template : Template file name to render the default view. If file name is a
                   relative path ('root.pt') the template is looked up in the active 
                   design or view module templatePath. The file name can also be a 
                   absolute dotted python name ('nive.cms.templates:root.pt').
        views    : List of object view definitions. Either nive.definitions.ViewConf 
                   or nive.definitions.ViewModuleConf.
        default  : Use this root as default root
        subtypes : Define possible subtypes. None=no objects allowed, *=all objects allowed, 
                   [IContainer,IPageElement]=list with interfaces of allowed objects.
        urlTraversal : if True the root name is allowed to be used in url traversal.
        workflowDisabled : Enable or disable workflow for this type.
        workflowID       : Workflow process id.
        events  : Register for one or multiple Application events. 
                  Register each event as e.g. Conf(event="run", callback=function).
        description : Description.

    Call RootConf().test() to verify configuration values.
    
    Interface: IRootConf
    """
    implements(IRootConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.context = "nive.components.objects.base.RootBase"
        self.extensions = None
        self.template = None
        self.default = True
        self.views = []
        self.subtypes = "*"
        self.workflowDisabled = True
        self.urlTraversal = True
        self.workflowID = ""
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)


    def __str__(self):
        return "%(name)s (%(id)s) implemented by %(context)s" % self

    def test(self):
        report = []
        # check id
        if not self.id:
            report.append((ConfigurationError, " RootConf.id is empty", self))
        # check context
        o = TryResolveName(self.context)
        if not o:
            report.append((ImportError, " for RootConf.context", self))
        # check extensions
        if self.extensions:
            for e in self.extensions:
                o = TryResolveName(e)
                if not o:
                    report.append((ImportError, " for ObjectConf.extensions", e))
        # check views
        if not type(self.views) in (ListType,TupleType):
            report.append((TypeError, " RootConf.views error: Not a list", self))
        else:
            for m in self.views:
                if hasattr(m, "test"):
                    report += m.test()
        return report        


class ViewModuleConf(baseConf):
    """
    View module configuration
    
    Used to group views and define module based default values for included views.
    View modules are registered and loaded on application level.
    
    Values ::

        *id    : Unique view module id as ascii string. Used to register and 
                 lookup the view module.
        name   : Display name 
        mainTemplate : The main template containing the slots to insert content.
        static : Static directory
        staticName : By default ViewModule.id is used as url name to map the static
                     directory. To change the default specify a different name here.  
        acl    : Pyramid security definitions added to nive.Application acls
        events  : Register for one or multiple Application events. 
                  Register each event as e.g. Conf(event="run", callback=function).
        description : optional description
    
    Included views ::

        views : a list of nive.definitions.ViewConf definitions. These views use the
                following default values.
        widgets : WidgetConf list for IToolboxWidgetConf and insertion points.

    View defaults, can be overridden by single views ::

        context      : default value for ViewModuleConf.views. 
        view         : default value for ViewModuleConf.views. 
        renderer     : default value for ViewModuleConf.views. 
        containment  : default value for ViewModuleConf.views. 
        permission   : default value for ViewModuleConf.views. 
        templates    : default value for ViewModuleConf.views, if renderers are used.
                       This refers to the directory templates with relative path are
                       looked up.

    Call ``ViewModuleConf().test()`` to verify configuration values.
        
    Interface: IViewModuleConf
    """
    implements(IViewModuleConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.static = ""
        self.staticName = None
        self.mainTemplate = None
        self.widgets = None
        self.acl = None
        self.description = u""
        
        # views and defaults
        self.context = None
        self.view = None
        self.containment = None    
        self.renderer = None    
        self.permission = None
        self.custom_predicates = None
        self.templates = ""
        self.views = []
        baseConf.__init__(self, copyFrom, **values)


    def __str__(self):
        return "%(id)s %(name)s" % self

    def test(self):
        report = []
        # check id
        if not self.id:
            report.append((ConfigurationError, " ViewModuleConf.id is empty", self))
        # check containment
        if self.containment:
            o = TryResolveName(self.containment)
            if not o:
                report.append((ImportError, " for ViewModuleConf.containment", self))
        # check view
        if self.view:
            o = TryResolveName(self.view)
            if not o:
                report.append((ImportError, " for ViewModuleConf.view", self))
        # check widgets
        if self.widgets:
            for w in self.widgets:
                report += w.test()
        # check defultTemplatePath
        #if self.defultTemplatePath:
        #    o = ResolveName(self.defultTemplatePath)
        #    if not o:
        #        report.append((ImportError, " ViewModuleConf.defultTemplatePath error", self))
        # check views
        if not type(self.views) in (ListType,TupleType):
            report.append((TypeError, " ViewModuleConf.views error: Not a list", self))
        else:
            for m in self.views:
                if hasattr(m, "test"):
                    report += m.test(viewModule=self)
        return report
        
        

class ViewConf(baseConf):
    """
    Configuration for a view definition
    
    Parameters refer mainly to pyramid.add_view. If the view is included in a ``ViewModuleConf``, defaults are 
    loaded from the module (see ``ViewModuleConf`` above). 

    Values ::

        *view       : the view class or function to be used 
        *name       : the url name used to call this view. May be empty as default view 
                      for object.
        attr        : if the view is mapped to a class, this is the function to be called
        context     : the object or interface this view is invoked for
        renderer    : renderer name or template path
        containment : load the view for this root context 
        permission  : permission required to access the view
        options     : additional view options supported by pyramid.add_view() as dict
        id          : (optional) unique view id as ascii string. Can be used to remove, 
                      change or replace a single view before the view is registered.
        description : (optional) description

    All parameters except *id* and *description* are passed to `pyramid.add_view() <http://docs.pylonsproject.org/projects/pyramid/en/latest/api/config.html#pyramid.config.Configurator.add_view>`_

    Call ``ViewConf().test()`` to verify configuration values.
    
    Interface: IViewConf
    """
    implements(IViewConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.attr = None
        self.name = None
        self.context = None
        self.view = None
        self.renderer = None
        self.containment = None
        self.permission = None
        self.options = {}
        self.custom_predicates = None
        self.settings = {}
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)


    def __str__(self):
        return "<%(name)s> %(id)s" % self

    def test(self, viewModule=None):
        if not viewModule:
            viewModule = ViewModuleConf()
        report = []
        # check context
        c = self.context or viewModule.context
        if c:
            o = TryResolveName(c)
            if not o:
                report.append((ImportError, " for ViewConf.context", self))
        # check view
        v = TryResolveName(self.view or viewModule.view)
        if not v:
            report.append((ImportError, " for ViewConf.view", self))
        # check renderer
        if self.renderer:
            import helper
            r = helper.ResolveAsset(self.renderer)
            try:
                r = helper.ResolveAsset(self.renderer)
            except:
                r = None
            if not r:
                report.append((ImportError, " for ViewConf.renderer template", self))
        # check attr
        #if self.attr:
        #    if not hasattr(v, self.attr):
        #        report.append((ImportError, " ViewConf.attr error", self, v))
        # check containment
        if self.containment:
            #!!! test containment for interface or class (not module)
            c = TryResolveName(self.containment)
            if not c:
                report.append((ImportError, " for ViewConf.containment", self))
        return report



class PortalConf(baseConf):
    """
    Portal configuration class. Contains all configurable options for the portal 
    (nive.Portal).
    
    Values ::

        portalDefaultUrl : redirect for portal root (/) requests
        loginUrl         : login form url 
        loginSuccessUrl  : redirect after succesfull login url 
        forbiddenUrl     : redirect for unauthorized requests
        logoutUrl        : logout function
        logoutSuccessUrl : redirect on succesfull logout
        accountUrl       : user account page
        favicon          : favicon asset path
        robots           : robots.txt contents
        
    Call ``PortalConf().test()`` to verify configuration values.
    
    Interface: IPortalConf
    """
    implements(IPortalConf)
    
    def __init__(self, copyFrom=None, **values):
        self.portalDefaultUrl = "/website/"
        self.loginUrl = "/userdb/udb/login"
        self.loginSuccessUrl = "/website/editor/"
        self.forbiddenUrl = "/userdb/udb/login"
        self.logoutUrl = "/userdb/udb/logout"
        self.logoutSuccessUrl = "/userdb/udb/login"
        self.accountUrl = "/userdb/udb/update"
        self.favicon = ""
        self.robots = u"""
User-agent: *
Disallow: /login
Disallow: /logout
Disallow: /update
        """
        baseConf.__init__(self, copyFrom, **values)

 
    def uid(self):
        # create a unique identifier for the configuration to support persistence
        return "nive."+self.__class__.__name__
    
    def test(self):
        report = []
        #check id
        return report
        


class ToolConf(baseConf):
    """
    Tool configuration 
    
    Values ::
    
        *id      : Unique tool id as ascii string. Used to register and lookup the tool.
        *context : Dotted python name or class reference used as factory. 
        *apply   : List of interfaces the tool is registered for. 
        data     : Tool data defined as nive.definitions.FieldConf list.
        values   : Values to override data.default on execution.
        views    : List of nive.definitions.ViewConf definitions. 
        mimetype : Mimetype of tool return stream
        hidden   : Hide in user interface.
        name     : Display name
        events   : Register for one or multiple Application events. 
                   Register each event as e.g. Conf(event="run", callback=function).
        description : Description
    
    Call ToolConf().test() to verify configuration values.
    
    Interface: IToolConf
    """
    implements(IToolConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.context = ""
        self.apply = None
        self.data = []
        self.values = {}
        self.views = []
        self.mimetype = ""
        self.hidden = False
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)

    def __call__(self, context):
        # for adapter registration
        return self, context

    
    def __str__(self):
        return "%(id)s (%(name)s) implemented by %(context)s" % self

    def test(self):
        report = []
        # check id
        if not self.id:
            report.append((ConfigurationError, " ToolConf.id is empty", self))
        # check context
        o = TryResolveName(self.context)
        if not o:
            report.append((ImportError, " for ToolConf.context", self))
        # check data
        if not type(self.data) in (ListType,TupleType):
            report.append((TypeError, " ToolConf.data error: Not a list", self))
        else:
            for m in self.data:
                if hasattr(m, "test"):
                    report += m.test()
        # check views
        if not type(self.views) in (ListType,TupleType):
            report.append((TypeError, " ToolConf.views error: Not a list", self))
        else:
            for m in self.views:
                if hasattr(m, "test"):
                    report += m.test()
        #check apply
        return report
        


class ModuleConf(baseConf):
    """
    Generic module definition for application extension

    Values ::

        *id     : Unique type id as ascii string.
        context : Dotted python name or class reference used as factory.
        name    : Display name.
        views   : List containing ViewConf or ViewModuleConf definitions. 
        events  : Register for one or multiple Application events. 
                  Register each event as e.g. Conf(event="run", callback=function).
        modules : Additional module configuration to be included.
        description : Description.

    Call ``ModuleConf().test()`` to verify configuration values.
    
    Interface: IModuleConf
    """
    implements(IModuleConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.context = ""
        self.views = []
        self.events = None
        self.modules = []
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)

    def __call__(self, context):
        return self

    
    def __str__(self):
        return "%(id)s (%(name)s)" % self

    def test(self):
        report = []
        # check id
        if not self.id:
            report.append((ConfigurationError, " ModuleConf.id is empty", self))
        # check context
        if self.context:
            o = TryResolveName(self.context)
            if not o:
                report.append((ImportError, " for ModuleConf.context", self))
        # check views
        if not type(self.views) in (ListType,TupleType):
            report.append((TypeError, " ModuleConf.views error: Not a list", self))
        else:
            for m in self.views:
                if hasattr(m, "test"):
                    report += m.test()
        # check modules
        for m in self.modules:
            if isinstance(m, basestring):
                o = TryResolveName(m)
                if not o:
                    report.append((ImportError, " AppConf.modules error: "+m, self, m))
            if hasattr(m, "test"):
                report += m.test()
        return report                
        

class WidgetConf(baseConf):
    """
    Configuration for a widget link
    
    Use this configuration to link existing views to plugin points
    in user interfaces. 
    Currently used for `nive.cms.cmsview` ToolboxWidget and EditorWidget

    Values ::    

        *apply      : List of interfaces of context objects this widget is used for.
        *viewmapper : View name in context to render widget contents.
        *widgetType : Plugin point to register this widget for (IToolboxWidgetConf or IEditorWidgetConf).
        *id         : Unique ascii string id to register this widget.
        name        : Widget name.
        sort        : Default sort for widgets as number.
        events      : Register for one or multiple Application events. 
                      Register each event as e.g. Conf(event="run", callback=function).
        description : Description.

    Call ``WidgetConf().test()`` to verify configuration values.
    
    Interface: IWidgetConf
    """
    implements(IWidgetConf)
    
    def __init__(self, copyFrom=None, **values):
        self.apply = None
        self.viewmapper = None
        self.widgetType = None
        self.name = None
        self.id = None
        self.sort = 100
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)

    def __call__(self, context):
        return self

    def __str__(self):
        return "%(id)s %(viewmapper)s %(widgetType)s" % self

    def test(self):
        report = []
        # check id
        if not self.id:
            report.append((ConfigurationError, " WidgetConf.id is empty", self))
        # check widgetType
        o = TryResolveName(self.widgetType)
        if not o:
            report.append((ImportError, " for WidgetConf.widgetType", self))
        #check view mapper
        if not self.viewmapper:
            report.append((ConfigurationError, " WidgetConf.viewmapper not defined", self))
        # check apply
        if not self.apply:
            report.append((ConfigurationError, " WidgetConf.apply not defined", self))
        else:
            for a in self.apply:
                o = TryResolveName(a)
                if not o:
                    report.append((ImportError, " for WidgetConf.apply", self))
        return report


class GroupConf(baseConf):
    """
    Configuration of a group or principal definition 

    Values ::

        id : Unique type id as ascii string with maximum 25 chars.
        name : Display name. 
        hidden : Hide in select lists in user interface.
        description : Description.

    Interface: IGroupConf
    """
    implements(IGroupConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.hidden = False
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)


class AclConf(baseConf):
    """
    Configuration of a acl definition 

    Values ::

        access : Access definition. Allow or Deny.
        principal : User/Group to map this acl to. Group or 'Everyone'. 
        permission : Permission name of this acl. custom string or 'ALL_PERMISSIONS'

    Interface: IGroupConf
    """
    implements(IAclConf)
    
    def __init__(self, copyFrom=None, **values):
        self.access = Allow
        self.principal = u""
        self.permission = u""
        baseConf.__init__(self, copyFrom, **values)


class CategoryConf(baseConf):
    """
    Configuration of a category

    id : Unique type id as ascii string with maximum 25 chars.
    name : Display name. 
    hidden : Hide in select lists in user interface.
    description : Description.
    """
    implements(ICategoryConf)
    
    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = u""
        self.hidden = False
        self.description = u""
        baseConf.__init__(self, copyFrom, **values)


class Conf(baseConf):
    """
    Configuration class with no predefined fields
    
    Interface: IConf
    """
    implements(IConf)
    
        


class ConfigurationError(Exception):
    pass
class ContainmentError(Exception):
    pass
class OperationalError(Exception):
    pass
class ProgrammingError(Exception):
    pass
class Warning(Exception):
    pass




def TryResolveName(name, base=None):
    if not name:
        return None
    if not isinstance(name, basestring):
        return name
    d = DottedNameResolver(base)
    try:
        return d.maybe_resolve(name)
    except:
        return None




# field type definitions ---------------------------------------------------------------

DataTypes = (
Conf(id="string",      name=_(u"String"),             description=u""),
Conf(id="number",      name=_(u"Number"),             description=u""),
Conf(id="float",       name=_(u"Float"),              description=u""),
Conf(id="bool",        name=_(u"Bool"),               description=u""),
Conf(id="htext",       name=_(u"HTML Text"),          description=u""),
Conf(id="text",        name=_(u"Text"),               description=u""),
Conf(id="code",        name=_(u"Code (Html, Javascript, Css)"), description=u""),
Conf(id="json",        name=_(u"Json"),               description=u""),
Conf(id="file",        name=_(u"File"),               description=u""),
Conf(id="date",        name=_(u"Date"),               description=u""),
Conf(id="datetime",    name=_(u"Datetime"),           description=u""),
Conf(id="list",        name=_(u"List"),               description=u""),
Conf(id="radio",       name=_(u"Radio Selection"),    description=u""),
Conf(id="mselection",  name=_(u"Multiple Selection"), description=u""),
Conf(id="mcheckboxes", name=_(u"Multiple Checkboxes"),description=u""),
Conf(id="lines",       name=_(u"Lines"),              description=u""),
Conf(id="email",       name=_(u"Email"),              description=u""),
Conf(id="url",         name=_(u"URL"),                description=u""),
Conf(id="urllist",     name=_(u"URL List"),           description=u""),
Conf(id="password",    name=_(u"Password"),           description=u""),
Conf(id="unit",        name=_(u"ID Reference"),       description=u""),
Conf(id="unitlist",    name=_(u"ID Reference List"),  description=u""),
Conf(id="timestamp",   name=_(u"Timestamp"),          description=u""),
Conf(id="binary",      name=_(u"Binary"),             description=u""),   # not supported as database field yet
)


# system meta fields -----------------------------------------------------------------------
SystemFlds = (
FieldConf(id="id",             datatype="number",    size=8,     default=0,     required=0,   readonly=1, name=_(u"ID")),
FieldConf(id="pool_type",      datatype="list",      size=35,    default=u"",   required=1,   readonly=1, name=_(u"Type")),
FieldConf(id="pool_unitref",   datatype="number",    size=8,     default=0,     required=0,   readonly=1, name=_(u"Container")),
FieldConf(id="pool_state",     datatype="number",    size=4,     default=1,     required=0,   readonly=0, name=_(u"State")),
FieldConf(id="pool_stag",      datatype="number",    size=4,     default=0,     required=0,   readonly=0, name=_(u"Select Number")),
FieldConf(id="pool_datatbl",   datatype="string",    size=35,    default=u"",   required=1,   readonly=1, name=_(u"Data Table Name")),
FieldConf(id="pool_dataref",   datatype="number",    size=8,     default=0,     required=1,   readonly=1, name=_(u"Data Table Reference")),
FieldConf(id="title",          datatype="string",    size=255,   default=u"",   required=0,   readonly=0, name=_(u"Title"), fulltext=True),
FieldConf(id="pool_sort",      datatype="number",    size=8,     default=0,     required=0,   readonly=0, name=_(u"Sort")),
FieldConf(id="pool_wfp",       datatype="list",      size=35,    default=u"",   required=0,   readonly=0, name=_(u"Workflow Process")),
FieldConf(id="pool_wfa",       datatype="list",      size=35,    default=u"",   required=0,   readonly=0, name=_(u"Workflow Activity")),
FieldConf(id="pool_category",  datatype="list",      size=35,    default=u"",   required=0,   readonly=0, name=_(u"Category")),
FieldConf(id="pool_filename",  datatype="string",    size=255,   default=u"",   required=0,   readonly=0, name=_(u"Filename")),
FieldConf(id="pool_create",    datatype="datetime",  size=100,   default=u"",   required=0,   readonly=1, name=_(u"Created")),
FieldConf(id="pool_change",    datatype="datetime",  size=100,   default=u"",   required=0,   readonly=1, name=_(u"Changed")),
FieldConf(id="pool_createdby", datatype="string",    size=35,    default=u"",   required=0,   readonly=1, name=_(u"Created by"), settings={u"relation":"userid"}),
FieldConf(id="pool_changedby", datatype="string",    size=35,    default=u"",   required=0,   readonly=1, name=_(u"Changed by"), settings={u"relation":"userid"}),
)
ReadonlySystemFlds = ("id", "pool_type", "pool_datatbl", "pool_unitref", "pool_dataref")

# base table structure -------------------------------------------------------------------

MetaTbl = "pool_meta"
FileTbl = "pool_files"
FulltextTbl = "pool_fulltext"
SystemTbl = "pool_sys"
LocalGroupsTbl = "pool_groups"
"""
``Structure`` defines the additional tables with settings for identity column and table fields. 
"""
Structure={
FileTbl: {"identity": "fileid",
          "fields": (
    FieldConf(id="fileid",     datatype="number",    size=8,     default=0,    required=0,   readonly=1, name=_(u"Unique ID")),
    FieldConf(id="id",         datatype="number",    size=8,     default=0,    required=0,   readonly=1, name=_(u"Object ID")),
    FieldConf(id="filekey",    datatype="string",    size=35,    default='',   required=1,   readonly=0, name=_(u"File key")),
    FieldConf(id="filename",   datatype="string",    size=255,   default='',   required=0,   readonly=0, name=_(u"Filename")),
    FieldConf(id="path",       datatype="string",    size=255,   default='',   required=0,   readonly=0, name=_(u"Internal file path")),
    FieldConf(id="size",       datatype="number",    size=8,     default=0,    required=0,   readonly=0, name=_(u"File size")),
    FieldConf(id="extension",  datatype="string",    size=5,     default='',   required=0,   readonly=0, name=_(u"File extension")),
    FieldConf(id="version",    datatype="string",    size=5,     default='',   required=0,   readonly=0, name=_(u"Version")),
)},
FulltextTbl: {"identity": None,
              "fields": (
    FieldConf(id="id",     datatype="number",    size=8,     default=0,    required=1,   readonly=1, name=_(u"Object ID")),
    FieldConf(id="text",   datatype="text",      size=0,     default="",   required=0,   readonly=0, name=_(u"Text")),
    FieldConf(id="files",  datatype="text",      size=0,     default="",   required=0,   readonly=0, name=_(u"Files")),
)},
SystemTbl: {"identity": None,
            "fields": (
    FieldConf(id="id",     datatype="string",    size=50,    default='',   required=0,   readonly=0, name=_(u"Key")),
    FieldConf(id="value",  datatype="text",      size=0,     default="",   required=0,   readonly=0, name=_(u"Value")),
    FieldConf(id="ts",     datatype="number",    size=8,     default="",   required=0,   readonly=0, name=_(u"Timestamp")),
)},
LocalGroupsTbl: {"identity": None,
                "fields": (
    FieldConf(id="id",     datatype="number",    size=8,     default=0,    required=1,   readonly=1, name=_(u"Object ID")),
    FieldConf(id="userid", datatype="string",    size=35,    default="",   required=1,   readonly=1, name=_(u"User name")),
    FieldConf(id="groupid",datatype="string",    size=20,    default="",   required=1,   readonly=1, name=_(u"Group assignment")),
)}
}

# select tags (pool_stag) ----------------------------------------------------------------------------

StagContainer = 0        # 0-9
StagPage = 10            # 10-19
StagPageElement = 20     # 20-29
StagRessource = 50       # 50-59
StagUser = 100           # 100-109

AllTypes = 1







import unittest

from pyramid import testing 
from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import render

from nive.cms.tests import db_app

from nive.cms.cmsview.view import Editor
from nive.security import User

from nive.cms.cmsview.sort import *


        
class tdbCutCopy(unittest.TestCase):
    
    def setUp(self):
        self.app = db_app.app()
        root = self.app.root("editor")
        user = User(u"test")
        user.groups.append("group:editor")
        self.page = db_app.create_page(root, user)
        self.text1 = db_app.create_text(root, user)
        self.text2 = db_app.create_text(root, user)
        pass
    
    def tearDown(self):
        root = self.app.root()
        user = User(u"test")
        user.groups.append("group:editor")
        try:
            root.Delete(self.page.id, user=user)
        except:
            pass
        try:
            root.Delete(self.text1.id, user=user)
        except:
            pass
        try:
            root.Delete(self.text2.id, user=user)
        except:
            pass
    

    def test_functions(self):
        # object functions
        self.text1.CanCopy()
        self.text1.CanPaste()
        
        # container
        self.page.CanCopy()
        self.page.CanPaste()

        root = self.app.root("editor")
        user = User(u"test")
        self.page.Paste([self.text1.id, self.text2.id], 0, user)
        new_texts = self.page.GetObjs()
        self.assert_(len(new_texts)==2)
        
        root.Move([text.id for text in new_texts], 0, user)
        ooo = self.page.GetObjs()
        #self.assertFalse(len(self.page.GetObjs()))
        self.assert_(root.Delete(new_texts[0].id, user))
        self.assert_(root.Delete(new_texts[1].id, user))
        


class tViewCutCopy(unittest.TestCase):

    def setUp(self):
        request = testing.DummyRequest()
        request._LOCALE_ = "en"
        self.request = request
        self.config = testing.setUp(request=request)
        self.app = db_app.app()
        self.app.Startup(self.config)
        self.root = self.app.root("editor")
        user = User(u"test")
        user.groups.append("group:editor")
        self.page = db_app.create_page(self.root, user)
        self.text1 = db_app.create_text(self.root, user)
        self.text2 = db_app.create_text(self.root, user)
        self.request.context = self.page

    def tearDown(self):
        user = User(u"test")
        self.root.Delete(self.page.id, user=user)
        self.app.Close()
        testing.tearDown()

    def test_views1(self):
        view = Editor(self.page, self.request)
        user = User(u"test")
        user.groups.append("group:editor")

        cp = view.SetCopyInfo("copy", [self.text1.id, self.text2.id], self.page)
        self.assert_(cp == ",".join(["copy", str(self.text1.id), str(self.text2.id)]))
        c1, c2 = view.GetCopyInfo()
        self.assert_(c1=="copy")
        self.assert_(len(c2)==2)
        self.assertFalse(view.ClipboardEmpty())
        view.DeleteCopyInfo()
        self.assert_(view.ClipboardEmpty())
        c1, c2 = view.GetCopyInfo()
        self.assert_(c1=="")
        self.assert_(len(c2)==0)

        self.request.GET = {"ids": [self.text1.id, self.text2.id]}
        self.assertRaises(HTTPFound, view.copy)
        c1, c2 = view.GetCopyInfo()
        self.assert_(c1=="copy")
        self.assert_(len(c2)==2)
        view.DeleteCopyInfo()

        self.assertRaises(HTTPFound, view.cut)
        c1, c2 = view.GetCopyInfo()
        self.assert_(c1=="cut")
        self.assert_(len(c2)==2)

        self.assertRaises(HTTPFound, view.copy)
        view.context = self.page
        self.assertRaises(HTTPFound, view.paste)
        new_texts = self.page.GetObjs()
        self.assert_(len(new_texts)==2)
        
        self.request.GET = {"ids": [text.id for text in new_texts]}
        view.context = self.page
        self.assertRaises(HTTPFound, view.cut)
        view.context = self.root
        self.assertRaises(HTTPFound, view.paste)
        ooo = self.page.GetObjs()
        self.assertFalse(len(self.page.GetObjs()))
        self.assert_(self.root.Delete(new_texts[0].id, user))
        self.assert_(self.root.Delete(new_texts[1].id, user))
        


